@@ -2,16 +2,17 @@
 # 
 #       Demo of some of the functionality for components.
 #       This is just a first sketch.
+#       
+# $Id: CarbonNotification.tcl,v 1.4 2004/12/01 15:15:40 matben Exp $
 
 namespace eval ::CarbonNotification:: {
     
 }
 
 proc ::CarbonNotification::Init { } {
-    global  tcl_platform
+    global  this
     
-    if {!([string equal $tcl_platform(platform) "unix"] && \
-      [string equal [tk windowingsystem] "aqua"])} {
+    if {![string equal $this(platform) "macosx"]} {
 	return
     }
     if {[catch {package require tclCarbonNotification}]} {
