@@ -4,9 +4,9 @@
 #       Complete rewrite by Mats Bengtsson   (matben@users.sourceforge.net)
 # 
 # Copyright (C) 2002-2004 Mats Bengtsson
-# This source file is distributed under the BSD licens.
+# This source file is distributed under the BSD license.
 # 
-# $Id: tree.tcl,v 1.33 2004/09/13 09:05:18 matben Exp $
+# $Id: tree.tcl,v 1.41 2004/11/06 08:15:25 matben Exp $
 # 
 # ########################### USAGE ############################################
 #
@@ -46,13 +46,15 @@
 #	-selectmode, selectMode, SelectMode                   (0|1)
 #	-selectoutline, selectOutline, SelectOutline
 #	-silent, silent, Silent                               (0|1)
+#	-sortcommand, sortCommand, SortCommand                tclProc
 #       -sortlevels, sortLevels, SortLevels
-#	-sortorder, sortOrder, SortOrder                      (decreasing|increasing)?
 #       -stripecolors, stripeColors, StripeColors
-#	-styleicons, styleIcons, StyleIcons                  (plusminus|triangle)
+#	-styleicons, styleIcons, StyleIcons                  
+#	                 (plusminus|plusminusbw|plusminusgray|triangle|crystal)
 #	-treecolor, treeColor, TreeColor                      color?
 #	-treedash, treeDash, TreeDash                         dash
 #	-width, width, Width
+#	-xmargin, xMargin, Margin
 #	-xscrollcommand, xScrollCommand, ScrollCommand
 #	-yscrollcommand, yScrollCommand, ScrollCommand
 #	
@@ -82,6 +84,7 @@
 #      -foreground    color
 #      -image         imageName
 #      -open          0|1
+#      -sortcommand   ""|tclProc
 #      -style         normal|bold|italic
 #      -tags
 #      -text
@@ -92,7 +95,7 @@
 #         v   itemPath
 #
 # *) a question mark (?) means that an empty list {} is also an option.
-#    -sortorder doesn't work when configure'ing.
+#    -sortcommand (on widget) doesn't work when configure'ing.
 # 
 # ########################### CHANGES ##########################################
 #
@@ -109,6 +112,7 @@
 #                   -stripecolors; 
 #                   items: -foreground, -backgroundbd,
 #                   reworked internals
+#       041029      major rework of drawing and a lot of other things
 
 package require Tcl 8.4
 package require colorutils
@@ -140,7 +144,7 @@ namespace eval tree {
 	    0x01, 0x01, 0x01, 0x01, 0xff, 0x01
 	};
     }
-    set widgetGlobals(openPM)   \
+    set widgetGlobals(openPMbw)   \
       [image create bitmap -data $data -maskdata $maskdata \
       -foreground black -background white]
     set data "#define closed_width 9\n#define closed_height 9"
@@ -150,10 +154,34 @@ namespace eval tree {
 	    0x11, 0x01, 0x01, 0x01, 0xff, 0x01
 	};
     }
-    set widgetGlobals(closePM)   \
+    set widgetGlobals(closePMbw)   \
       [image create bitmap -data $data -maskdata $maskdata \
       -foreground black -background white]
     
+    set widgetGlobals(openPM) [image create photo -data {
+	R0lGODdhCQAJAKIAAP//////wsLCwsLCibS0tFOJwgAAAAAAACwAAAAACQAJ
+	AAADHUi1XAowgiUjrYKavXOBQSh4YzkuAkEMrKI0C5EAADs=
+    }]    
+    set widgetGlobals(closePM) [image create photo -data {
+	R0lGODdhCQAJAKIAAP//////wsLCwsLCibS0tFOJwgAAAAAAACwAAAAACQAJ
+	AAADIEi1XAowghVNpNACQY33XAEFRiCEp2Cki0AQQ6wozUIkADs=
+    }]
+
+    set widgetGlobals(openPMgray) [image create photo -data {
+	R0lGODdhCwALAMQAAP////39/fPz8+zs7OPj49vb29TU1CQkJCEhIRwcHBoa
+	GhQUFBMTEw4ODggICAQEBAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
+	AAAAAAAAAAAAAAAAAAAAAAAAACwAAAAACwALAAAFOyAkjqRonEVBrIOJrsQw
+	PNArD4LgQGmcC4FAg0coBRkQFjAYVCR9zCAiiYsGDxDZMjqFPByNhQJxQCRC
+	ADs=
+    }]     
+    set widgetGlobals(closePMgray) [image create photo -data {
+	R0lGODdhCwALAMQAAP////7+/vn5+fX19fHx8e3t7enp6eTk5N3d3dTU1CEh
+	IRoaGhQUFA0NDQsLCwUFBQAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
+	AAAAAAAAAAAAAAAAAAAAAAAAACwAAAAACwALAAAFQOCijEvJMM6jBGwbCILq
+	ssIwyG5NFNAyCztDzwUZFA4HCINWOyIQytdgd0AkEpCGzlC9Yh3AAtd6hTzO
+	kLQ6HQIAOw==
+    }]
+
     set widgetGlobals(idir) [image create photo ::tree::idir -data {
 	R0lGODlhEAAQAPMAMf////oG+Pj4+Pj4ALi4uHh4eAAAAAAAAAAAAAAAAAAA
 	AAAAAAAAAAAAAAAAAAAAACH5BAEAAAEALAAAAAAQABAAAAQ+MMhJq70418K5
@@ -209,7 +237,23 @@ namespace eval tree {
 	FuraxfshA4UKXmuc8AAB4V0XHzp4tTFig2DHeENYteFhQULHMm7cAOF5QEAA
 	Ow==
     }] 
-        
+
+    # Crystal arrows.
+    set widgetGlobals(openCrystal) [image create photo -data {
+	R0lGODlhDAAKAPQAMfEI5KTK/Hyq9HSq9Fyi/FyGzFSa/FSS9FSO9FSCzEyK
+	9ESG7ESC7DyC7Dx67Dx65Dx25DR67DR25DRy5DRqzCxm1CRazAQ2rAQCxAAA
+	AAAAAAAAAAAAAAAAAAAAAAAAACH5BAEAAAAALAAAAAAMAAoAAAU7ICAKREkU
+	YgoMQctc6hgcyAPHLKJAN5AMA0NAwZhcjrBCS9FoSAKvVGVBdUSMMQojcu2p
+	LN2YCuMFhAAAOw==
+    }]
+    set widgetGlobals(closeCrystal) [image create photo -data {
+	R0lGODlhCgAMAPQAMfEI5Lza/KzO/Iy2/Iyy9IS+/Hy2/Hyy/HSy/HSq/Gyu
+	/Gym/GSm/GSi9FyW9FSS9FSO9Eya/EyG9ESG7Dx67AQ2rAAAAAAAAAAAAAAA
+	AAAAAAAAAAAAAAAAAAAAAAAAACH5BAEAAAAALAAAAAAKAAwAAAU7ICAC0WiK
+	UXWOUaCKTBM3QfPShYEszaMygUJC0YhAJhXaIdFwHCkVhmA6eEiggIpWC8Ge
+	IoTXV3wiA0IAOw==
+    }]
+
     # Let them be accesible from the outside. 
     set ::tree::idir            $widgetGlobals(idir)
     set ::tree::ifile           $widgetGlobals(ifile)
@@ -272,12 +316,13 @@ proc ::tree::Init { } {
 	-selectoutline       {selectOutline        SelectOutline       }
 	-silent              {silent               Silent              }
 	-sortlevels          {sortLevels           SortLevels          }
-	-sortorder           {sortOrder            SortOrder           }
+	-sortcommand         {sortCommand          SortCommand         }
 	-stripecolors        {stripeColors         StripeColors        }
 	-styleicons          {styleIcons           StyleIcons          }
 	-treecolor           {treeColor            TreeColor           }
 	-treedash            {treeDash             TreeDash            }
 	-width               {width                Width               }
+	-xmargin             {xMargin              Margin              }
 	-xscrollcommand      {xScrollCommand       ScrollCommand       }
 	-yscrollcommand      {yScrollCommand       ScrollCommand       }
     }
@@ -291,8 +336,9 @@ proc ::tree::Init { } {
   
     # The legal widget commands.
     set widgetCommands {cget children closetree configure delitem   \
-      getselection getcanvas isitem itemconfigure labelat newitem opentree   \
-      setselection xview yview}
+      getselection getcanvas isitem itemconfigure labelat loweritem \
+      newitem opentree   \
+      raiseitem setselection xview yview}
 
     # Options for this widget
     option add *Tree.background            white           widgetDefault
@@ -307,7 +353,7 @@ proc ::tree::Init { } {
     option add *Tree.highlightColor        black           widgetDefault
     option add *Tree.highlightThickness    3               widgetDefault
     option add *Tree.height                100             widgetDefault
-    option add *Tree.indention             0               widgetDefault
+    option add *Tree.indention             8               widgetDefault
     option add *Tree.itemBackgoundBd       0               widgetDefault
     option add *Tree.openImage             ""              widgetDefault
     option add *Tree.openCommand           {}              widgetDefault
@@ -325,6 +371,7 @@ proc ::tree::Init { } {
     option add *Tree.treeColor             gray50          widgetDefault
     option add *Tree.treeDash              {}              widgetDefault
     option add *Tree.width                 100             widgetDefault
+    option add *Tree.xMargin               10              widgetDefault
     option add *Tree.xScrollCommand        {}              widgetDefault
     option add *Tree.yScrollCommand        {}              widgetDefault
     
@@ -348,9 +395,9 @@ proc ::tree::Init { } {
 	windows {
 	    option add *Tree.selectBackground systemHighlight     widgetDefault
 	    option add *Tree.selectForeground systemHighlightText widgetDefault
-	    set widgetGlobals(font)             {Arial 8 normal}
-	    set widgetGlobals(fontbold)         {Arial 8 bold}
-	    set widgetGlobals(fontitalic)       {Arial 8 italic}
+	    set widgetGlobals(font)             {{MS Sans Serif} 8 normal}
+	    set widgetGlobals(fontbold)         {{MS Sans Serif} 8 bold}
+	    set widgetGlobals(fontitalic)       {{MS Sans Serif} 8 italic}
 	}
 	macintosh {
 	    option add *Tree.selectBackground systemHighlight     widgetDefault
@@ -518,7 +565,7 @@ proc ::tree::tree {w args} {
     array set metricsDirArr [font metrics $options(-fontdir)]
     set linespace [expr {$metricsArr(-linespace) > $metricsDirArr(-linespace)} ? \
       $metricsArr(-linespace) : $metricsDirArr(-linespace)]
-    set yline [expr $linespace + 5]
+    set yline [expr {$linespace + 5}]
     set priv(yline) [expr {$yline < 17} ? 17 : $yline]
     
     # Provide some default bindings.
@@ -748,7 +795,7 @@ proc ::tree::WidgetProc {w command args} {
 		    set vlist {}
 		    foreach {key val} [array get state "*:tags"] {
 			if {[string equal $val $ftag]} {
-			    set ind [expr [string last ":tags" $key] - 1]
+			    set ind [expr {[string last ":tags" $key] - 1}]
 			    set uid [string range $key 0 $ind]
 			    if {[info exists uid2v($uid)]} {
 				lappend vlist $uid2v($uid)
@@ -776,12 +823,18 @@ proc ::tree::WidgetProc {w command args} {
 	labelat {
 	    set result [eval LabelAt $w $args]
 	}
+	loweritem {
+	    set result [eval RaiseLowerItem $w lower $args]
+	}
 	newitem {
 	    set result [eval NewItem $w $args]
 	}
 	opentree {
 	    set result [eval OpenTree $w $args]
 	}
+	raiseitem {
+	    set result [eval RaiseLowerItem $w raise $args]
+	}
 	setselection {
 	    set result [eval SetSelection $w $args]
 	}
@@ -897,7 +950,7 @@ proc ::tree::Configure {w args} {
 		    RemoveSelection $w
 		}
 	    }
-	    -sortorder {
+	    -sortcommand {
 		
 		if {![string equal $newValue $oldValue]} {
 		    
@@ -966,21 +1019,41 @@ proc ::tree::ConfigureIcons {w} {
     upvar ::tree::${w}::options options
     upvar ::tree::${w}::priv priv
 
-    if {[string equal $options(-styleicons) "plusminus"]} {
-	set priv(imclose) $widgetGlobals(closePM)	
-	set priv(imopen)  $widgetGlobals(openPM)
-    } elseif {[string equal $options(-styleicons) "triangle"]} {
-	set priv(imclose) $widgetGlobals(closeMac)	
-	set priv(imopen)  $widgetGlobals(openMac)
-    } else {
-	return -code error "unrecognized value \"$options(-styleicons)\" for -styleicons"
+    switch -- $options(-styleicons) {
+	plusminus {
+	    set priv(imclose) $widgetGlobals(closePM)	
+	    set priv(imopen)  $widgetGlobals(openPM)
+	}
+	plusminusbw {
+	    set priv(imclose) $widgetGlobals(closePMbw)	
+	    set priv(imopen)  $widgetGlobals(openPMbw)
+	}
+	plusminusgray {
+	    set priv(imclose) $widgetGlobals(closePMgray)	
+	    set priv(imopen)  $widgetGlobals(openPMgray)
+	}
+	triangle {
+	    set priv(imclose) $widgetGlobals(closeMac)	
+	    set priv(imopen)  $widgetGlobals(openMac)
+	}
+	crystal {
+	    set priv(imclose) $widgetGlobals(closeCrystal)	
+	    set priv(imopen)  $widgetGlobals(openCrystal)
+	}
+	"" {
+	    set priv(imclose) ""
+	    set priv(imopen)  ""
+	}
+	default {
+	    return -code error "unrecognized value \"$options(-styleicons)\" for -styleicons"
+	}
     }
 
     # Let any -closeimage or -openimage override.
-    if {[string length $options(-closeimage)]} {
+    if {$options(-closeimage) != ""} {
 	set priv(imclose) $options(-closeimage)	
     }
-    if {[string length $options(-openimage)]} {
+    if {$options(-openimage) != ""} {
 	set priv(imopen) $options(-openimage)	
     }
 }
@@ -1096,6 +1169,13 @@ proc ::tree::ConfigureItem {w v args} {
 	    -open {
 		set result $state($uid:open)
 	    }
+	    -sortcommand {
+		if {[info exists state($uid:scmd)]} {
+		    set result $state($uid:scmd)
+		} else {
+		    set result ""
+		}
+	    }
 	    -style {
 		set result $state($uid:style)
 	    }
@@ -1164,6 +1244,11 @@ proc ::tree::SetItemOptions {w v args} {
 	    -open {
 		set state($uid:open) $val
 	    }
+	    -sortcommand {
+		set state($uid:scmd) $val
+		set state($uid:children)  \
+		  [eval $state($uid:scmd) [list $state($uid:children)]]
+	    }
 	    -style {
 		if {[regexp {(normal|bold|italic)} $val]} {
 		    set state($uid:style) $val
@@ -1246,20 +1331,24 @@ proc ::tree::NewItem {w v args} {
     } else {
 	lappend state($uidDir:children) $tail
     }
-    if {[llength $options(-sortorder)]} {
+    if {[llength $options(-sortcommand)]} {
 	set doSort 1
 	if {[llength $options(-sortlevels)]} {
-	    set sort [lindex $options(-sortlevels) [expr [llength $v] - 1]]
+	    set sort [lindex $options(-sortlevels) [expr {[llength $v] - 1}]]
 	    if {[string equal $sort "0"]} {
 		set doSort 0
 	    }
 	}
 	if {$doSort} {
-	    set state($uidDir:children)   \
-	      [lsort -$options(-sortorder) $state($uidDir:children)]
+	    set state($uidDir:children)  \
+	      [eval $options(-sortcommand) [list $state($uidDir:children)]]
 	}
     }
-
+    if {[info exists state($uidDir:scmd)] && ($state($uidDir:scmd) != "")} {
+	set state($uidDir:children)  \
+	  [eval $state($uidDir:scmd) [list $state($uidDir:children)]]
+    }
+    
     # Make fresh uid now that we know it's ok to create it.
     set uid [incr vuid]
     set v2uid($v)   $uid
@@ -1335,6 +1424,7 @@ proc ::tree::DelItem {w v args} {
 	    unset state($uid:text)
 	    unset state($uid:bg)
 	    unset state($uid:tags)
+	    unset -nocomplain state($uid:scmd)
 	    unset -nocomplain state($uid:tag)
 	    unset -nocomplain state($uid:ctags)
 	    set dir [lrange $v 0 end-1]
@@ -1355,6 +1445,58 @@ proc ::tree::DelItem {w v args} {
     BuildWhenIdle $w
 }
 
+# ::tree::RaiseLowerItem --
+#
+#       Move an item up or down in the list of childrens.
+#       
+# Arguments:
+#       w       the widget path.
+#       what    "raise" or "lower".
+#       v       the item as a list.
+#       which   above or below tailname.
+#       
+# Results:
+#       none.
+
+proc ::tree::RaiseLowerItem {w what v {which ""}} {
+    
+    variable widgetGlobals
+    upvar ::tree::${w}::state state
+    upvar ::tree::${w}::v2uid v2uid
+    upvar ::tree::${w}::uid2v uid2v
+
+    set v [NormList $v]   
+    set dir  [lrange $v 0 end-1]
+    set tail [lindex $v end]
+    if {![info exists v2uid($dir)]} {
+	return -code error "parent item \"$dir\" is missing"
+    }
+    set uidDir $v2uid($dir)
+    if {$which == ""} {
+	if {$what == "raise"} {
+	    set idxin 0
+	} else {
+	    set idxin end
+	}
+    } else {
+	if {$what == "raise"} {
+	    set idxin [lsearch $state($uidDir:children) $which]
+	} else {
+	    set idxin [lsearch $state($uidDir:children) $which]
+	    incr idxin
+	}
+    }
+    set tmp $state($uidDir:children)
+    set idx [lsearch $tmp $tail]
+    if {$idx >= 0} {
+	set tmp [lreplace $tmp $idx $idx]
+    }
+    set state($uidDir:children) [linsert $tmp $idxin $tail]	
+    
+    BuildWhenIdle $w
+    return ""
+}
+
 # These procedures are only used for handling the Button bind commands.
 
 proc ::tree::SelectCmd {w x y} {
@@ -1409,13 +1551,13 @@ proc ::tree::SetSelection {w v} {
     # Modify our view so selection is visible.
     if {[string length $options(-yscrollcommand)] && [winfo ismapped $w]} {
 	set coords [$widgets(canvas) coords $state(selidx)]
-	set midysel [expr ([lindex $coords 1] + [lindex $coords 3])/2]
+	set midysel [expr {([lindex $coords 1] + [lindex $coords 3])/2}]
 	set scrollregion [$widgets(canvas) cget -scrollregion]
 	set scrollheight [lindex $scrollregion 3]
 	set yview [$widgets(canvas) yview]
-	set ytop [expr [lindex $yview 0] * $scrollheight]
-	set ybot [expr [lindex $yview 1] * $scrollheight]
-	if {$midysel < [expr $ytop + 15]} {
+	set ytop [expr {[lindex $yview 0] * $scrollheight}]
+	set ybot [expr {[lindex $yview 1] * $scrollheight}]
+	if {$midysel < [expr {$ytop + 15}]} {
 	    
 	    # Be sure to never scroll past the top.
 	    if {$midysel < 40} {
@@ -1423,7 +1565,7 @@ proc ::tree::SetSelection {w v} {
 	    } else {
 		$widgets(canvas) yview scroll -1 pages
 	    }
-	} elseif {$midysel > [expr $ybot - 15]} {
+	} elseif {$midysel > [expr {$ybot - 15}]} {
 	    $widgets(canvas) yview scroll 1 pages
 	}
     }
@@ -1499,8 +1641,11 @@ proc ::tree::Build {w} {
 
     set can $widgets(canvas)
     
-    # Standard indention from center line to text start.
-    set priv(xindent) [expr [image width $priv(imopen)]/2 + 6]
+    # Standard indention from center line to icon or text start.
+    set priv(xindent) $options(-xmargin)
+    if {$priv(imopen) != ""} {
+	incr priv(xindent) [expr {[image width $priv(imopen)]/2}]
+    }
     foreach col $options(-stripecolors) {
 	set priv($col:stripelight) [::colorutils::getlighter $col]
 	set priv($col:stripedark)  [::colorutils::getdarker $col]
@@ -1521,7 +1666,11 @@ proc ::tree::Build {w} {
     # Keeps track of top y coords to draw.
     set state(y) 0
     set state(i) 0
-    BuildLayer $w {} [expr [image width $priv(imopen)]/2 + 4]
+    set xin 4
+    if {$priv(imopen) != ""} {
+	incr xin [expr {[image width $priv(imopen)]/2}]
+    }
+    BuildLayer $w {} $xin
     
     # At this stage the display list is almost completely mixed up. Reorder!
     $can lower ttreev ttreeh
@@ -1575,12 +1724,12 @@ proc ::tree::DrawBackgroundImage {w} {
 # Arguments:
 #       w       the widget path.
 #       v       the item as a list.
-#       in      indention in pixels.
+#       xin     x coordinate for the vertical tree line.
 #       
 # Results:
 #       new tree layer is drawn.
 
-proc ::tree::BuildLayer {w v in} {
+proc ::tree::BuildLayer {w v xin} {
 
     variable widgetGlobals
     upvar ::tree::${w}::widgets widgets
@@ -1589,7 +1738,7 @@ proc ::tree::BuildLayer {w v in} {
     upvar ::tree::${w}::priv priv
     upvar ::tree::${w}::v2uid v2uid
     
-    Debug 2 "::tree::BuildLayer v=$v, in=$in"
+    Debug 2 "::tree::BuildLayer v=$v, xin=$xin"
 
     set can $widgets(canvas)
     set hasTree 0
@@ -1598,7 +1747,7 @@ proc ::tree::BuildLayer {w v in} {
     set yTreeOff $widgetGlobals(yTreeOff)
 
     set treeCol $options(-treecolor)
-    set indention [expr $priv(xindent) + $options(-indention)]
+    set indention $options(-indention)
     if {[string length $treeCol]} {
 	set hasTree 1
     }
@@ -1607,19 +1756,22 @@ proc ::tree::BuildLayer {w v in} {
     } else {
 	set vx $v
     }
-    set uid $v2uid($v)
-    set y      $state(y)
-    set ystart $y
-    set yline  $priv(yline)
-    set yoff   [expr $yline/2]
-    set ycent  [expr $y + $yoff]
+    set uid     $v2uid($v)
+    set xoff    $priv(xindent)
+    set y       $state(y)
+    set ystart  $y
+    set yline   $priv(yline)
+    set yoff    [expr {$yline/2}]
+    set ycent   [expr {$y + $yoff}]
     set scrollwidth $options(-scrollwidth)
     set fg          $options(-foreground)
     set stripecols  $options(-stripecolors)
     set stripelen   [llength $stripecols]
     set itembd      $options(-itembackgroundbd)
-
-    Debug 3 "\tuid=$uid"
+    set treedash    $options(-treedash)
+    set pyjamascol  $options(-pyjamascolor)
+    
+    Debug 3 "\t uid=$uid"
     
     # Loop through all childrens.
     foreach c $state($uid:children) {
@@ -1636,23 +1788,23 @@ proc ::tree::BuildLayer {w v in} {
 	    set isDir 1
 	}
 	set y     $state(y)
-	set ycent [expr $y + $yoff]
-	set ylow  [expr $y + $yline]
-	set ylow1 [expr $ylow - 1]
+	set ycent [expr {$y + $yoff}]
+	set ylow  [expr {$y + $yline}]
+	set ylow1 [expr {$ylow - 1}]
 	set state($uidc:y) $y
 	
 	# Any background color?
 	set bgi ""
-	if {[string length $state($uidc:bg)]} {
-	    set bgi $state($uidc:bg)
+	if {$state($uidc:bg) != ""} {
+	    set bgi     $state($uidc:bg)
 	    set bglight $state($uidc:bglight)
 	    set bgdark  $state($uidc:bgdark)
-	} elseif {[string length $stripecols]} {
-	    set bgi [lindex $stripecols [expr $state(i) % $stripelen]]
+	} elseif {$stripecols != ""} {
+	    set bgi     [lindex $stripecols [expr {$state(i) % $stripelen}]]
 	    set bglight $priv($bgi:stripelight)
 	    set bgdark  $priv($bgi:stripedark)
 	}
-	if {[string length $bgi]} {
+	if {$bgi != ""} {
 	    
 	    # Draw plain background.
 	    $can create rectangle 0 $y $scrollwidth $ylow \
@@ -1660,10 +1812,10 @@ proc ::tree::BuildLayer {w v in} {
 	    
 	    # Any borders.
 	    set bdi 0
-	    if {[string length $state($uidc:bd)]} {
+	    if {$state($uidc:bd) != ""} {
 		set bdi $state($uidc:bd)
 	    } 
-	    if {($bdi == 0) && [string length $itembd]} {
+	    if {($bdi == 0) && ($itembd != "")} {
 		set bdi $itembd
 	    }
 	    if {$bdi > 0} {
@@ -1678,34 +1830,33 @@ proc ::tree::BuildLayer {w v in} {
 	incr state(y) $yline
 	
 	# Any pyjamas lines?
-	if {[llength $options(-pyjamascolor)] > 0} {
-	    $can create line 0 $ylow 4000 $ylow  \
-	      -fill $options(-pyjamascolor) -tags tpyj	    
+	if {$pyjamascol != ""} {
+	    $can create line 0 $ylow 4000 $ylow -fill $pyjamascol -tags tpyj	    
 	}
 	
 	# Tree lines?
 	if {$hasTree} {
-	    $can create line $in $ycent [expr $in + $indention - 4] $ycent \
-	      -fill $treeCol -tags ttreeh -dash $options(-treedash)
+	    $can create line $xin $ycent [expr {$xin + $xoff - 4}] $ycent \
+	      -fill $treeCol -tags ttreeh -dash $treedash
 	}
-	set icon $state($uidc:icon)
-	set text $state($uidc:text)
 	
-	# The 'x' means selectable!
-	#set taglist [list x $state($uidc:tags)]
+	# The 'x' tag means selectable!
 	set taglist x
 	set ids {}
-	set x [expr $in + $indention]
 	if {[info exists state($uidc:ctags)]} {
 	    lappend taglist $state($uidc:ctags)
 	}
 	
-	if {[string length $icon] > 0} {
+	set x [expr {$xin + $xoff}]
+	set icon $state($uidc:icon)
+	set text $state($uidc:text)
+	
+	if {$icon != ""} {
 	    set id [$can create image $x $ycent -image $icon -anchor w \
 	      -tags $taglist]
 	    set state(v:$id) $vxc
 	    lappend ids $id
-	    incr x [expr [image width $icon] + 6]
+	    incr x [expr {[image width $icon] + 6}]
 	}
 	if {[info exists state($uidc:style)]} {
 	    set style $state($uidc:style)
@@ -1723,7 +1874,7 @@ proc ::tree::BuildLayer {w v in} {
 	}
 	set id [$can create text $x $ycent -text $text -font $itemFont \
 	  -anchor w -tags $taglist -fill $fgi]
-	    lappend ids $id
+	lappend ids $id
 	if {[info exists state($uidc:text2)]} {
 	    set id2 [$can create text 140 $ycent -text $state($uidc:text2)  \
 	      -font $options(-font) -anchor w -fill $fgi]
@@ -1737,20 +1888,26 @@ proc ::tree::BuildLayer {w v in} {
 	# Do we have a directory here?
 	if {$isDir} {
 	    if {$state($uidc:open)} {
-		set id [$can create image $in $ycent -image $imopen -tags topen]
+		if {$imopen != ""} {
+		    set id [$can create image $xin $ycent -image $imopen -tags topen]
+		}
 		if {$hasChildren} {
 		
 		    # Call this recursively. 
 		    # The number here is the directory offset in x.
-		    BuildLayer $w $vxc [expr $in + $indention]
+		    BuildLayer $w $vxc [expr {$xin + $indention}]
 		}
 	    } else {
-		set id [$can create image $in $ycent -image $imclose -tags tclose]
+		if {$imclose != ""} {
+		    set id [$can create image $xin $ycent -image $imclose \
+		      -tags tclose]
+		}
 	    }
 	}
     }
     if {$hasTree} {
-	$can create line $in $ystart $in [expr $ycent + $yTreeOff]  \
+	$can create line $xin [expr {$ystart - $yoff}] \
+	  $xin [expr {$ycent + $yTreeOff}]  \
 	  -fill $treeCol -tags ttreev -dash $options(-treedash)
     }
 }
@@ -1880,8 +2037,8 @@ proc ::tree::DrawSelection {w} {
     set bbox [$can bbox $state($uid:tag)]
     if {[llength $bbox] == 4} {
 	set bbox [list \
-	  [expr [lindex $bbox 0] - 2] [expr $state($uid:y) + 1]  \
-	  [expr [lindex $bbox 2] + 2] [expr $state($uid:y) + $priv(yline)]]
+	  [expr {[lindex $bbox 0] - 2}] [expr {$state($uid:y) + 1}]  \
+	  [expr {[lindex $bbox 2] + 2}] [expr {$state($uid:y) + $priv(yline)}]]
 	
 	set id [eval {$can create rectangle} $bbox {-fill $options(-selectbackground) \
 	  -outline $options(-selectoutline) -dash $options(-selectdash)}]
@@ -1965,8 +2122,8 @@ proc ::tree::NextLabel {w direction} {
 
     set selBbox [$widgets(canvas) bbox $state(selidx)]
     if {[llength $selBbox] > 0} {
-	set yMid [expr $direction * $priv(yline) + \
-	  ([lindex $selBbox 1] + [lindex $selBbox 3])/2.0]
+	set yMid [expr {$direction * $priv(yline) + \
+	  ([lindex $selBbox 1] + [lindex $selBbox 3])/2.0}]
 	foreach id [$widgets(canvas) find overlapping 10 $yMid 200 $yMid] {
 	    if {[$widgets(canvas) type $id] == "text"} {
 		if {[info exists state(v:$id)]} {
