@@ -5,12 +5,10 @@
 #      
 #  Copyright (c) 2004  Mats Bengtsson
 #  
-# $Id: Emoticons.tcl,v 1.17 2004/09/28 13:50:17 matben Exp $
-
+# $Id: Emoticons.tcl,v 1.26 2004/12/20 15:16:45 matben Exp $
 
 package provide Emoticons 1.0
 
-
 namespace eval ::Emoticons:: {
 
     # Define all hooks for preference settings.
@@ -82,178 +80,22 @@ proc ::Emoticons::Init { } {
     ::Emoticons::SetPermanentSet $jprefs(emoticonSet)
 }
 
-# OUTDATED!
-
-proc ::Emoticons::MakeDarkProjectSet { } {
-    global  this
-    
+proc ::Emoticons::Exists {word} {
     variable smiley
-    variable smileyExp
-    variable smileyLongNames
-
-    ::Debug 2 "::Emoticons::MakeDarkProjectSet"
     
-    # Smiley icons. The "short" types.
-    foreach {key name} {
-	":-)"          classic 
-	":-("          sad 
-	":-0"          shocked 
-	";-)"          wink
-	";("           cry
-	":o"           embarrassed
-	":D"           grin
-	"x)"           knocked
-	":|"           normal
-	":S"           puzzled
-	":p"           silly
-	":O"           shocked
-	":x"           speechless} {
-	    set imSmile($name) [image create photo -format gif  \
-	      -file [file join $this(imagePath) smileys "smiley-${name}.gif"]]
-	    set smiley($key) $imSmile($name)
-    }
-	
-    # Duplicates:
-    foreach {key name} {
-	":)"           classic 
-	";)"           wink} {
-	    set smiley($key) $imSmile($name)
-    }
-    
-    # Seems unused...
-    set smileyExp {(}
-    foreach key [array names smiley] {
-	append smileyExp "$key|"
-    }
-    set smileyExp [string trimright $smileyExp "|"]
-    append smileyExp {)}
-    regsub  {[)(|]} $smileyExp {\\\0} smileyExp
-    
-    # The "long" smileys are treated differently; only loaded when needed.
-    set smileyLongNames {
-	:alien:
-	:angry:
-	:bandit:
-	:beard:
-	:bored:
-	:calm:
-	:cat:
-	:cheeky:
-	:cheerful:
-	:chinese:
-	:confused:
-	:cool:
-	:cross-eye:
-	:cyclops:
-	:dead:
-	:depressed:
-	:devious:
-	:disappoin:
-	:ditsy:
-	:dog:
-	:ermm:
-	:evil:
-	:evolved:
-	:gasmask:
-	:glasses:
-	:happy:
-	:hurt:
-	:jaguar:
-	:kommie:
-	:laugh:
-	:lick:
-	:mad:
-	:nervous:
-	:ninja:
-	:ogre:
-	:old:
-	:paranoid:
-	:pirate:
-	:ponder:
-	:puzzled:
-	:rambo:
-	:robot:
-	:eek:
-	:shocked:
-	:smiley:
-	:sleeping:
-	:smoker:
-	:surprised:
-	:tired:
-	:vampire:
+    if {[info exists smiley($word)]} {
+	return 1
+    } else {
+	return 0
     }
 }
-    
-# Emoticons::Parse --
-# 
-#       Parses text into a list with every second element an image create command.
-#       
-# Arguments:
-#       str         the text string, tcl special chars already protected
-#       
-# Results:
-#       A list {str {image create ..} ?str {image create ..} ...?}
 
-proc ::Emoticons::Parse {str} {
-    global  this
-    
+proc ::Emoticons::Make {w word} {
     variable smiley
-    variable smileyLongNames
     
-    set _begin {( |^)}
-    set _end {( |$)}
-    
-    # Protect Tcl special characters, quotes included.
-    regsub -all {([][$\\{}"])} $str {\\\1} str
-        
-    # Since there are about 60 smileys we need to be economical here.    
-    # Check first if there are any short smileys.
-    
-    # Protect all  *regexp*  special characters. Regexp hell!!!
-    # Carefully embrace $smile since may contain ; etc.
-    # Reqire one space on each side, replaced with -padx 6.
-    
-    foreach smile [array names smiley] {
-	set sub "\} \{image create end -image $smiley($smile) -padx 6 -name \{$smile\}\} \{"
-	    
-	# Protect all regexp special characters!
-	regsub -all {[][$^);(\|\?\*\\]} $smile {\\\0} smileExp
-	if {[catch {
-	    regsub -all ${_begin}${smileExp}${_end} $str $sub str
-	}]} {
-	    puts "--->smile=$smile, smileExp=$smileExp"
-	}
+    if {[info exists smiley($word)]} {
+	$w image create end -image $smiley($word) -name $word
     }
-    
-    # Now check for any "long" names, such as :angry: :cool: etc.
-    # OUTDATED!
-    if {[info exists smileyLongNames]} {
-	set candidateList {}
-	set ndx 0
-	
-	while {[regexp -start $ndx -indices -- {:[a-zA-Z]+:} $str ind]} {
-	    set ndx [lindex $ind 1]
-	    set candidate [string range $str [lindex $ind 0] [lindex $ind 1]]
-	    if {[lsearch $smileyLongNames $candidate] >= 0} {
-		lappend candidateList $candidate
-		
-		# Load image if not done that.
-		if {![info exists smileyLongIm($candidate)]} {
-		    set fileName "smiley-[string trim $candidate :].gif"
-		    set smileyLongIm($candidate) [image create photo -format gif  \
-		      -file [file join $this(imagePath) smileys $fileName]]	    
-		}
-	    }
-	}
-	if {[llength $candidateList]} {
-	    regsub -all {\\|&} $str {\\\0} str
-	    foreach smile $candidateList {
-		set sub "\} \{image create end -image $smileyLongIm($smile) -name $smile\} \{"
-		regsub -all $smile $str $sub str
-	    }
-	}
-    }
-    return "\{$str\}"
 }
 
 proc ::Emoticons::GetAllSets { } {
@@ -420,7 +262,8 @@ proc ::Emoticons::ParseIcon {name dir xmllist} {
 	    # If we rely on QuickTimeTcl here we cannot be in vfs.
 	    set f [file join $dir $object]
 	    if {$priv(needtmp)} {
-		set tmp [file join $this(tmpPath) $object]
+		#set tmp [file join $this(tmpPath) $object]
+		set tmp [::tfileutils::tempfile $this(tmpPath) [file rootname $object]]
 		file copy -force $f $tmp
 		set f $tmp
 	    }
@@ -580,7 +423,7 @@ proc ::Emoticons::MenuButton {w args} {
     
     eval {::Emoticons::BuildMenu $wmenu} $args
 
-    bind $w <Button-1> [list [namespace current]::PostMenu $wmenu %X %Y]
+    bind $w <Button-1> [list [namespace current]::PostMenu $w $wmenu %X %Y]
     return $w
 }
 
@@ -645,9 +488,11 @@ proc ::Emoticons::SortLength {str1 str2} {
     return [expr {[string length $str1] < [string length $str2]} ? -1 : 1]
 }
 
-proc ::Emoticons::PostMenu {m x y} {
+proc ::Emoticons::PostMenu {w m x y} {
 
-    tk_popup $m [expr int($x)] [expr int($y)]
+    if {[string equal [$w cget -state] "normal"]} {
+	tk_popup $m [expr int($x)] [expr int($y)]
+    }
 }
 
 proc ::Emoticons::InsertSmiley {wtext imname name} {
@@ -680,7 +525,7 @@ proc ::Emoticons::InsertTextLegend {w name args} {
 	$w insert insert "\n"
     }
     
-    $w insert insert "\tImage\tText\n"
+    $w insert insert "\t[mc Image]\t[mc Text]\n"
     
     # Smileys:
     foreach ind [lsort -dictionary [array names tmpiconsInv $name,*]] {
@@ -737,8 +582,8 @@ proc ::Emoticons::BuildPrefsPage {wpage} {
     if {[llength $allSets] == 0} {
 	set allSets None
     }
-    label $wpage.l -text "The selected iconset will take action when you save"
-    pack $wpage.l -side top -anchor w -padx 8 -pady 4
+    label $wpage.l -text [mc preficonsel]
+    pack  $wpage.l -side top -anchor w -padx 8 -pady 4
     eval {tk_optionMenu $wpop [namespace current]::tmpSet} $allSets
     labelframe $wfr -labelwidget $wpop -padx 6 -pady 4
     pack $wfr -side top -anchor w -padx 8 -pady 4 -fill both -expand 1
@@ -772,7 +617,7 @@ proc ::Emoticons::PopCmd {name1 name2 op} {
 	if {[catch {
 	    ::Emoticons::LoadTmpIconSet $state($var,path)
 	} err]} {
-	    tk_messageBox -icon error -title [mc Error] \
+	    ::UI::MessageBox -icon error -title [mc Error] \
 	      -message "Failed loading iconset $var. $err" \
 	      -parent [winfo toplevel $wpreftext]
 	    set priv(havezip) 0
