@@ -1,16 +1,16 @@
 #  Jabber.tcl ---
 #  
 #      This file is part of The Coccinella application. 
-#      It implements the "glue" between the whiteboard and jabberlib.
 #      
 #  Copyright (c) 2001-2004  Mats Bengtsson
 #  
 #  See the README file for license, bugs etc.
 #
-# $Id: Jabber.tcl,v 1.110 2004/10/09 13:21:55 matben Exp $
+# $Id: Jabber.tcl,v 1.123 2004/12/21 15:14:42 matben Exp $
 
 package require balloonhelp
 package require browse
+package require chasearrows
 package require combobox
 package require disco
 package require http 2.3
@@ -20,6 +20,7 @@ package require sha1pure
 package require tinyfileutils
 package require tree
 package require uriencode
+package require wavelabel
 
 # We should have some component mechanism that lets packages load themselves.
 package require Agents
@@ -44,14 +45,14 @@ package require Privacy
 package require Profiles
 package require Register
 package require Roster
+package require Rosticons
 package require Search
+package require Status
 package require Subscribe
 package require VCard
 
-
 package provide Jabber 1.0
 
-
 namespace eval ::Jabber:: {
     global  this prefs
     
@@ -73,6 +74,8 @@ namespace eval ::Jabber:: {
     set jstate(sock) {}
     set jstate(ipNum) {}
     set jstate(inroster) 0
+    
+    # This is our own status (presence/show).
     set jstate(status) "unavailable"
     
     # Server port actually used.
@@ -99,28 +102,6 @@ namespace eval ::Jabber:: {
     variable jidPublic
     set jidPublic(haveit) 0
     
-    # Mappings from <show> element to displayable text and vice versa.
-    # chat away xa dnd
-    variable mapShowElemToText
-    variable mapShowTextToElem
-    
-    array set mapShowElemToText [list \
-      [mc mAvailable]       available  \
-      [mc mAway]            away       \
-      [mc mChat]            chat       \
-      [mc mDoNotDisturb]    dnd        \
-      [mc mExtendedAway]    xa         \
-      [mc mInvisible]       invisible  \
-      [mc mNotAvailable]    unavailable]
-    array set mapShowTextToElem [list \
-      available       [mc mAvailable]     \
-      away            [mc mAway]          \
-      chat            [mc mChat]          \
-      dnd             [mc mDoNotDisturb]  \
-      xa              [mc mExtendedAway]  \
-      invisible       [mc mInvisible]     \
-      unavailable     [mc mNotAvailable]]
-        
     # Array that maps namespace (ns) to a descriptive name.
     variable nsToText
     array set nsToText {
@@ -171,6 +152,24 @@ namespace eval ::Jabber:: {
 	caps            http://coccinella.sourceforge.net/protocol/caps
     }
     
+    # Standard jabber (xmpp + JEP) protocol namespaces.
+    variable xmppxmlns
+    array set xmppxmlns {
+	amp         "http://jabber.org/protocol/amp"
+	caps        "http://jabber.org/protocol/caps"
+	disco       "http://jabber.org/protocol/disco"
+	disco,info  "http://jabber.org/protocol/disco#info"
+	disco,items "http://jabber.org/protocol/disco#items"
+	ibb         "http://jabber.org/protocol/ibb"
+	muc         "http://jabber.org/protocol/muc"
+	muc,admin   "http://jabber.org/protocol/muc#admin"
+	muc,owner   "http://jabber.org/protocol/muc#owner"
+	muc,user    "http://jabber.org/protocol/muc#user"
+    }
+
+    variable capsExtList
+    set capsExtList {ftrans}
+    
     # Standard xmlns supported. Components add their own.
     variable clientxmlns
     set clientxmlns {
@@ -290,8 +289,8 @@ proc ::Jabber::FactoryDefaults { } {
     set jprefs(autoBrowseConference) 0
     
     # Dialog pane positions.
-    set prefs(paneGeom,$wDlgs(jchat)) {0.75 0.25}
-    set prefs(paneGeom,$wDlgs(jinbox)) {0.5 0.5}
+    set prefs(paneGeom,$wDlgs(jchat))    {0.75 0.25}
+    set prefs(paneGeom,$wDlgs(jinbox))   {0.5 0.5}
     set prefs(paneGeom,groupchatDlgVert) {0.8 0.2}
     set prefs(paneGeom,groupchatDlgHori) {0.8 0.2}
     
@@ -362,7 +361,6 @@ proc ::Jabber::SetUserPreferences { } {
       [list ::Jabber::jprefs(port)             jprefs_port              $jprefs(port)]  \
       [list ::Jabber::jprefs(sslport)          jprefs_sslport           $jprefs(sslport)]  \
       [list ::Jabber::jprefs(agentsServers)    jprefs_agentsServers     $jprefs(agentsServers)]  \
-      [list ::Jabber::jprefs(browseServers)    jprefs_browseServers     $jprefs(browseServers)]  \
       [list ::Jabber::jserver(profile)         jserver_profile          $jserver(profile)      userDefault] \
       [list ::Jabber::jserver(profile,selected) jserver_profile_selected $jserver(profile,selected) userDefault] \
       ]
@@ -429,16 +427,6 @@ proc ::Jabber::GetMyStatus { } {
     return $jstate(status)
 }
 
-proc ::Jabber::GetStatusText {status} {    
-    variable mapShowTextToElem
-    
-    if {[info exists mapShowTextToElem($status)]} {
-	return $mapShowTextToElem($status)
-    } else {
-	return ""
-    }
-}
-
 proc ::Jabber::JlibCmd {args} {
     variable jstate
     
@@ -495,7 +483,7 @@ proc ::Jabber::Init { } {
     ::Debug 2 "::Jabber::Init"
     
     # Make the roster object.
-    set jstate(roster) [::roster::roster ::Jabber::Roster::PushProc]
+    set jstate(roster) [::roster::roster ::Roster::PushProc]
     
     # Check if we need to set any auto away options.
     set opts {}
@@ -517,7 +505,7 @@ proc ::Jabber::Init { } {
     } $opts]
 
     # Register handlers for various iq elements.
-    $jstate(jlib) iq_register get jabber:iq:version      ::Jabber::ParseGetVersion
+    $jstate(jlib) iq_register get jabber:iq:version    ::Jabber::ParseGetVersion
     $jstate(jlib) iq_register get $coccixmlns(servers) ::Jabber::ParseGetServers
     
     # Set the priority order of groupchat protocols.
@@ -574,40 +562,39 @@ proc ::Jabber::MessageCallback {jlibName type args} {
     
     ::Debug 2 "::Jabber::MessageCallback type=$type, args='$args'"
     
-    array set attrArr {-body ""}
-    array set attrArr $args
+    array set argsArr {-body ""}
+    array set argsArr $args
     
-    set from $attrArr(-from)
+    set from $argsArr(-from)
         
     switch -- $type {
 	error {
 	    
 	    # We must check if there is an error element sent along with the
 	    # body element. In that case the body element shall not be processed.
-	    if {[info exists attrArr(-error)]} {
-		set errcode [lindex $attrArr(-error) 0]
-		set errmsg [lindex $attrArr(-error) 1]
+	    if {[info exists argsArr(-error)]} {
+		set errcode [lindex $argsArr(-error) 0]
+		set errmsg [lindex $argsArr(-error) 1]
 		
-		tk_messageBox -title [mc Error] \
-		  -message [FormatTextForMessageBox \
-		  [mc jamesserrsend $attrArr(-from) $errcode $errmsg]]  \
+		::UI::MessageBox -title [mc Error] \
+		  -message [mc jamesserrsend $argsArr(-from) $errcode $errmsg] \
 		  -icon error -type ok		
 	    }
 	    eval {::hooks::run newErrorMessageHook} $args
 	}
 	chat {
-	    eval {::hooks::run newChatMessageHook $attrArr(-body)} $args
+	    eval {::hooks::run newChatMessageHook $argsArr(-body)} $args
 	}
 	groupchat {
-	    eval {::hooks::run newGroupChatMessageHook $attrArr(-body)} $args
+	    eval {::hooks::run newGroupChatMessageHook $argsArr(-body)} $args
 	}
 	headline {
-	    eval {::hooks::run newHeadlineMessageHook $attrArr(-body)} $args
+	    eval {::hooks::run newHeadlineMessageHook $argsArr(-body)} $args
 	}
 	default {
 	    
 	    # Normal message. Handles whiteboard stuff as well.
-	    eval {::hooks::run newMessageHook $attrArr(-body)} $args
+	    eval {::hooks::run newMessageHook $argsArr(-body)} $args
 	}
     }
 }
@@ -632,8 +619,8 @@ proc ::Jabber::PresenceCallback {jlibName type args} {
     
     ::Debug 2 "::Jabber::PresenceCallback type=$type, args='$args'"
     
-    array set attrArr $args
-    set from $attrArr(-from)
+    array set argsArr $args
+    set from $argsArr(-from)
     
     switch -- $type {
 	subscribe {
@@ -650,11 +637,11 @@ proc ::Jabber::PresenceCallback {jlibName type args} {
 	    
 	    if {[regexp {^(service|gateway)/.*} $jidtype]} {
 		$jstate(jlib) send_presence -to $from -type "subscribed"
-		$jstate(jlib) roster_set $from ::Jabber::Subscribe::ResProc
+		$jstate(jlib) roster_set $from ::Subscribe::ResProc
 		
 		set subtype [lindex [split $jidtype /] 1]
-		set typename [::Jabber::Roster::GetNameFromTrpt $subtype]
-		tk_messageBox -title [mc {Transport Suscription}] \
+		set typename [::Roster::GetNameFromTrpt $subtype]
+		::UI::MessageBox -title [mc {Transport Subscription}] \
 		  -icon info -type ok \
 		  -message [mc jamesstrptsubsc $typename]
 	    } else {
@@ -691,12 +678,12 @@ proc ::Jabber::PresenceCallback {jlibName type args} {
 			set msg [mc jamessautoreject $from]
 		    }
 		    ask {
-			eval {::Jabber::Subscribe::NewDlg $from} $args
+			eval {::Subscribe::NewDlg $from} $args
 		    }
 		}
 		if {$msg != ""} {
-		    tk_messageBox -title [mc Info] -icon info -type ok \
-		      -message [FormatTextForMessageBox $msg]			      
+		    ::UI::MessageBox -title [mc Info] -icon info -type ok \
+		      -message $msg
 		}
 		
 		# Auto subscribe to subscribers to me.
@@ -704,46 +691,45 @@ proc ::Jabber::PresenceCallback {jlibName type args} {
 		    
 		    # Explicitly set the users group.
 		    if {[string length $jprefs(subsc,group)]} {
-			$jstate(jlib) roster_set $from ::Jabber::Subscribe::ResProc \
+			$jstate(jlib) roster_set $from ::Subscribe::ResProc \
 			  -groups [list $jprefs(subsc,group)]
 		    }
 		    $jstate(jlib) send_presence -to $from -type "subscribe"
 		    set msg [mc jamessautosubs $from]
-		    tk_messageBox -title [mc Info] -icon info -type ok \
-		      -message [FormatTextForMessageBox $msg]			  
+		    ::UI::MessageBox -title [mc Info] -icon info -type ok \
+		      -message $msg
 		}
 	    }
 	}
 	subscribed {
-	    tk_messageBox -title [mc Subscribed] -icon info -type ok  \
-	      -message [FormatTextForMessageBox [mc jamessallowsub $from]]
+	    ::UI::MessageBox -title [mc Subscribed] -icon info -type ok  \
+	      -message [mc jamessallowsub $from]
 	}
 	unsubscribe {	    
 	    if {$jprefs(rost,rmIfUnsub)} {
 		
 		# Remove completely from our roster.
-		$jstate(jlib) roster_remove $from ::Jabber::Roster::PushProc
-		tk_messageBox -title [mc Unsubscribe] \
+		$jstate(jlib) roster_remove $from ::Roster::PushProc
+		::UI::MessageBox -title [mc Unsubscribe] \
 		  -icon info -type ok  \
-		  -message [FormatTextForMessageBox [mc jamessunsub $from]]
+		  -message [mc jamessunsub $from]
 	    } else {
 		
 		$jstate(jlib) send_presence -to $from -type "unsubscribed"
-		tk_messageBox -title [mc Unsubscribe] -icon info -type ok  \
-		  -message [FormatTextForMessageBox [mc jamessunsubpres $from]]
+		::UI::MessageBox -title [mc Unsubscribe] -icon info -type ok  \
+		  -message [mc jamessunsubpres $from]
 		
 		# If there is any subscription to this jid's presence.
 		set sub [$jstate(roster) getsubscription $from]
 		if {[string equal $sub "both"] ||  \
 		  [string equal $sub "to"]} {
 		    
-		    set ans [tk_messageBox -title [mc Unsubscribed]  \
+		    set ans [::UI::MessageBox -title [mc Unsubscribed]  \
 		      -icon question -type yesno -default yes \
-		      -message [FormatTextForMessageBox  \
-		      [mc jamessunsubask $from $from]]]
+		      -message [mc jamessunsubask $from $from]]
 		    if {$ans == "yes"} {
 			$jstate(jlib) roster_remove $from \
-			  ::Jabber::Roster::PushProc
+			  ::Roster::PushProc
 		    }
 		}
 	    }
@@ -755,31 +741,29 @@ proc ::Jabber::PresenceCallback {jlibName type args} {
 	    set sub [$jstate(roster) getsubscription $from]
 	    if {$sub == "none"} {
 		set msg [mc jamessfailedsubsc $from]
-		if {[info exists attrArr(-status)]} {
-		    append msg " Status message: $attrArr(-status)"
+		if {[info exists argsArr(-status)]} {
+		    append msg " Status message: $argsArr(-status)"
 		}
-		tk_messageBox -title [mc {Subscription Failed}]  \
+		::UI::MessageBox -title [mc {Subscription Failed}]  \
 		  -icon info -type ok  \
-		  -message [FormatTextForMessageBox $msg]
+		  -message $msg
 		if {$jprefs(rost,rmIfUnsub)} {
 		    
 		    # Remove completely from our roster.
-		    $jstate(jlib) roster_remove $from ::Jabber::Roster::PushProc
+		    $jstate(jlib) roster_remove $from ::Roster::PushProc
 		}
 	    } else {		
-		tk_messageBox -title [mc Unsubscribed]  \
+		::UI::MessageBox -title [mc Unsubscribed]  \
 		  -icon info -type ok  \
-		  -message [FormatTextForMessageBox  \
-		  [mc jamessunsubscribed $from]]
+		  -message [mc jamessunsubscribed $from]
 	    }
 	}
 	error {
-	    foreach {errcode errmsg} $attrArr(-error) break		
+	    foreach {errcode errmsg} $argsArr(-error) break		
 	    set msg [mc jamesserrpres $errcode $errmsg]
 	    if {$prefs(talkative)} {
-		tk_messageBox -icon error -type ok  \
-		  -title [mc {Presence Error}] \
-		  -message [FormatTextForMessageBox $msg]	
+		::UI::MessageBox -icon error -type ok  \
+		  -title [mc {Presence Error}] -message $msg
 	    }
 	    ::Jabber::AddErrorLog $from $msg
 	}
@@ -829,15 +813,14 @@ proc ::Jabber::ClientProc {jlibName what args} {
 	    # Disconnect. This should reset both wrapper and XML parser!
 	    ::Jabber::DoCloseClientConnection
 	    
-	    tk_messageBox -icon error -type ok  \
+	    ::UI::MessageBox -icon error -type ok  \
 	      -message [mc jamessconnbroken]
 	}
 	away - xaway {
 	    
 	    set tm [clock format [clock seconds] -format "%H:%M:%S"]
-	    set ans [tk_messageBox -icon info -type yesno -default yes \
-	      -message [FormatTextForMessageBox \
-	      [mc jamessautoawayset $tm]]]
+	    set ans [::UI::MessageBox -icon info -type yesno -default yes \
+	      -message [mc jamessautoawayset $tm]]
 	    if {$ans == "yes"} {
 		::Jabber::SetStatus available
 	    }
@@ -850,8 +833,8 @@ proc ::Jabber::ClientProc {jlibName what args} {
 	    } else {
 		set msg "Receieved a fatal error. The connection is closed."
 	    }
-	    tk_messageBox -title [mc {Fatal Error}] -icon error -type ok \
-	      -message [FormatTextForMessageBox $msg]
+	    ::UI::MessageBox -title [mc {Fatal Error}] -icon error -type ok \
+	      -message $msg
 	}
 	xmlerror {
 	    
@@ -865,16 +848,15 @@ proc ::Jabber::ClientProc {jlibName what args} {
 		set msg "Receieved a fatal XML parsing error.\
 		  The connection is closed down."
 	    }
-	    tk_messageBox -title [mc {Fatal Error}] -icon error -type ok \
-	      -message [FormatTextForMessageBox $msg]
+	    ::UI::MessageBox -title [mc {Fatal Error}] -icon error -type ok \
+	      -message $msg
 	}
 	networkerror {
 	    
 	    # Disconnect. This should reset both wrapper and XML parser!
 	    ::Jabber::DoCloseClientConnection
-	    tk_messageBox -title [mc {Network Error}] \
-	      -message [FormatTextForMessageBox $argsArr(-body)] \
-	      -icon error -type ok	    
+	    ::UI::MessageBox -title [mc {Network Error}] \
+	      -message $argsArr(-body) -icon error -type ok	    
 	}
     }
     return $ishandled
@@ -993,8 +975,7 @@ proc ::Jabber::IqSetGetCallback {method jlibName type theQuery} {
 		  and with message: $errmsg"
 	    }
 	}
-	tk_messageBox -icon error -type ok -title [mc Error] -message \
-	  [FormatTextForMessageBox $msg]
+	::UI::MessageBox -icon error -type ok -title [mc Error] -message $msg
     }
 }
 
@@ -1211,6 +1192,11 @@ proc ::Jabber::SetStatus {type args} {
 	}	
     }
     
+    # General presence should not have a 'to' attribute.
+    if {![info exists argsArr(-to)]} {
+	set jstate(status) $type
+    }
+    
     # Trap network errors.
     # It is somewhat unclear here if we should have a type attribute
     # when sending initial presence, see XMPP 5.1.
@@ -1220,8 +1206,7 @@ proc ::Jabber::SetStatus {type args} {
 	
 	# Close down?	
 	DoCloseClientConnection
-	tk_messageBox -title [mc Error] -icon error -type ok \
-	  -message [FormatTextForMessageBox $err]
+	::UI::MessageBox -title [mc Error] -icon error -type ok -message $err
     } else {
 	
 	eval {::hooks::run setPresenceHook $type} $args
@@ -1281,16 +1266,23 @@ proc ::Jabber::CreateCoccinellaPresElement { } {
 proc ::Jabber::CreateCapsPresElement { } {
     global  prefs
     variable coccixmlns
+    variable capsExtList
+    variable xmppxmlns
 
-    set capsxmlns "http://jabber.org/protocol/caps"
     set node $coccixmlns(caps)
-    set ext "ftrans"
-    set xmllist [wrapper::createtag c \
-      -attrlist [list xmlns $capsxmlns node $node ver $prefs(fullVers) ext $ext]]
+    set xmllist [wrapper::createtag c -attrlist \
+      [list xmlns $xmppxmlns(caps) node $node ver $prefs(fullVers) ext $capsExtList]]
 
     return $xmllist
 }
 
+proc ::Jabber::RegisterCapsExtKey {key} {    
+    variable capsExtList
+    
+    lappend capsExtList $key
+    set capsExtList [lsort -unique $capsExtList]
+}
+
 # Jabber::SetStatusWithMessage --
 #
 #       Dialog for setting user's status with message.
@@ -1331,7 +1323,7 @@ proc ::Jabber::SetStatusWithMessage { } {
     pack $frtop -side top -fill x -padx 4 -pady 4
     set i 0
     foreach val {available chat away xa dnd invisible} {
-	label ${frtop}.l${val} -image [::Jabber::Roster::GetPresenceIconFromKey $val]
+	label ${frtop}.l${val} -image [::Roster::GetPresenceIconFromKey $val]
 	radiobutton ${frtop}.${val} -text [mc jastat${val}]  \
 	  -variable [namespace current]::show -value $val
 	grid ${frtop}.l${val} -sticky e -column 0 -row $i -padx 4 -pady 3
@@ -1393,45 +1385,10 @@ proc ::Jabber::BtSetStatus {w} {
     set allText [string trim [$wtext get 1.0 end] " \n"]
     
     # Set present status.
-    set jstate(status) $show
     SetStatus $show -status $allText    
     set finishedStat 1
     destroy $w
 }
-
-# Jabber::ParseAndInsertText --
-#
-#       Parses for smileys and url's and insert into text widget.
-#       
-# Arguments:
-#       w           text widget
-#       str         raw text to process
-#       
-# Results:
-#       none.
-
-proc ::Jabber::ParseAndInsertText {w str tag linktag} {
-    
-    # Smileys.
-    set cmdList [::Emoticons::Parse $str]
-    
-    # Http links.
-    set textCmdList {}
-    foreach {txt icmd} $cmdList {
-	set httpCmd [::Text::ParseHttpLinks $txt $tag $linktag]
-	if {$icmd == ""} {
-	    eval lappend textCmdList $httpCmd
-	} else {
-	    eval lappend textCmdList $httpCmd [list $icmd]
-	}
-    }
-
-    # Insert into text widget.
-    foreach cmd $textCmdList {
-	eval {$w} $cmd
-    }
-    $w insert end "\n"
-}
     
 # Jabber::GetAnyDelayElem --
 #
@@ -1489,15 +1446,14 @@ proc ::Jabber::GetLastResult {from silent jlibname type subiq} {
 	set msg [mc jamesserrlastactive $from [lindex $subiq 1]]
 	::Jabber::AddErrorLog $from $msg	    
 	if {!$silent} {
-	    tk_messageBox -title [mc Error] -icon error -type ok \
-	      -message [FormatTextForMessageBox $msg]
+	    ::UI::MessageBox -title [mc Error] -icon error -type ok \
+	      -message $msg
 	}
     } else {
 	array set attrArr [wrapper::getattrlist $subiq]
 	if {![info exists attrArr(seconds)]} {
-	    tk_messageBox -title [mc {Last Activity}] -icon info  \
-	      -type ok -message [FormatTextForMessageBox \
-	      [mc jamesserrnotimeinfo $from]]
+	    ::UI::MessageBox -title [mc {Last Activity}] -icon info  \
+	      -type ok -message [mc jamesserrnotimeinfo $from]
 	} else {
 	    set secs [expr [clock seconds] - $attrArr(seconds)]
 	    set uptime [clock format $secs -format "%a %b %d %H:%M:%S"]
@@ -1519,8 +1475,8 @@ proc ::Jabber::GetLastResult {from silent jlibname type subiq} {
 	    } else {
 		set msg1 [mc jamessuptime]
 	    }
-	    tk_messageBox -title [mc {Last Activity}] -icon info  \
-	      -type ok -message [FormatTextForMessageBox "$msg1 $uptime. $msg"]
+	    ::UI::MessageBox -title [mc {Last Activity}] -icon info  \
+	      -type ok -message "$msg1 $uptime. $msg"
 	}
     }
 }
@@ -1551,9 +1507,8 @@ proc ::Jabber::GetTimeResult {from silent jlibname type subiq} {
 	  "We received an error when quering its time info.\
 	  The error was: [lindex $subiq 1]"	    
 	if {!$silent} {
-	    tk_messageBox -title [mc Error] -icon error -type ok \
-	      -message [FormatTextForMessageBox \
-	      [mc jamesserrtime $from [lindex $subiq 1]]]
+	    ::UI::MessageBox -title [mc Error] -icon error -type ok \
+	      -message [mc jamesserrtime $from [lindex $subiq 1]]
 	}
     } else {
 	
@@ -1571,8 +1526,8 @@ proc ::Jabber::GetTimeResult {from silent jlibname type subiq} {
 	} else {
 	    set msg "unknown"
 	}
-	tk_messageBox -title [mc {Local Time}] -icon info -type ok -message \
-	  [FormatTextForMessageBox [mc jamesslocaltime $from $msg]]
+	::UI::MessageBox -title [mc {Local Time}] -icon info -type ok \
+	  -message [mc jamesslocaltime $from $msg]
     }
 }
 
@@ -1610,9 +1565,8 @@ proc ::Jabber::GetVersionResult {from silent jlibname type subiq} {
 	::Jabber::AddErrorLog $from  \
 	  [mc jamesserrvers $from [lindex $subiq 1]]
 	if {!$silent} {
-	    tk_messageBox -title [mc Error] -icon error -type ok \
-	      -message [FormatTextForMessageBox \
-	      [mc jamesserrvers $from [lindex $subiq 1]]]
+	    ::UI::MessageBox -title [mc Error] -icon error -type ok \
+	      -message [mc jamesserrvers $from [lindex $subiq 1]]
 	}
     } else {
 	set w .jvers[incr uidvers]
@@ -1710,7 +1664,7 @@ proc ::Jabber::ParseGetVersion {jlibname from subiq args} {
       [wrapper::createtag os      -chdata $os]]
     set xmllist [wrapper::createtag query -subtags $subtags  \
       -attrlist {xmlns jabber:iq:version}]
-     eval {$jstate(jlib) send_iq "result" $xmllist} $opts
+     eval {$jstate(jlib) send_iq "result" [list $xmllist]} $opts
 
     # Tell jlib's iq-handler that we handled the event.
     return 1
@@ -1752,7 +1706,7 @@ proc ::Jabber::ParseGetServers  {jlibname from subiq args} {
       [wrapper::createtag ip -chdata $ip -attrlist $attrhttpd]]
     set xmllist [wrapper::createtag query -subtags $subtags  \
       -attrlist [list xmlns $coccixmlns(servers)]]
-     eval {$jstate(jlib) send_iq "result" $xmllist} $opts
+     eval {$jstate(jlib) send_iq "result" [list $xmllist]} $opts
     
      # Tell jlib's iq-handler that we handled the event.
     return 1
@@ -1884,8 +1838,7 @@ proc ::Jabber::Passwd::Doit {w} {
     ::Debug 2 "::Jabber::Passwd::Doit"
 
     if {![string equal $validate $password]} {
-	tk_messageBox -type ok -icon error  \
-	  -message [FormatTextForMessageBox [mc jamesspasswddiff]]
+	::UI::MessageBox -type ok -icon error -message [mc jamesspasswddiff]
 	return
     }
     set finished 1
@@ -1905,9 +1858,8 @@ proc ::Jabber::Passwd::ResponseProc {jlibName type theQuery} {
 	set errcode [lindex $theQuery 0]
 	set errmsg [lindex $theQuery 1]
 	set msg 
-	tk_messageBox -title [mc Error] -icon error -type ok \
-	  -message [FormatTextForMessageBox  \
-	  [mc jamesspasswderr $errcode $errmsg]] \
+	::UI::MessageBox -title [mc Error] -icon error -type ok \
+	  -message [mc jamesspasswderr $errcode $errmsg]
     } else {
 		
 	# Make sure the new password is stored in our profiles.
@@ -1919,8 +1871,8 @@ proc ::Jabber::Passwd::ResponseProc {jlibName type theQuery} {
 		eval {::Profiles::Set {}} $spec
 	    }
 	}
-	tk_messageBox -title [mc {New Password}] -icon info -type ok \
-	  -message [FormatTextForMessageBox [mc jamesspasswdok]]
+	::UI::MessageBox -title [mc {New Password}] -icon info -type ok \
+	  -message [mc jamesspasswdok]
     }
 }
 
@@ -1934,7 +1886,7 @@ proc ::Jabber::LoginLogout { } {
     if {[::Jabber::IsConnected]} {
 	::Jabber::DoCloseClientConnection
     } else {
-	::Jabber::Login::Dlg
+	::Login::Dlg
     }    
 }
 
