@@ -1,16 +1,15 @@
 #  Login.tcl ---
 #  
 #      This file is part of The Coccinella application. 
-#      It implements the Roster GUI part.
+#      It implements functions for logging in at different application levels.
 #      
 #  Copyright (c) 2001-2004  Mats Bengtsson
 #  
-# $Id: Login.tcl,v 1.46 2004/09/28 13:50:18 matben Exp $
+# $Id: Login.tcl,v 1.57 2004/12/14 14:08:45 matben Exp $
 
 package provide Login 1.0
 
-namespace eval ::Jabber::Login:: {
-    global  wDlgs
+namespace eval ::Login:: {
     
     variable server
     variable username
@@ -20,11 +19,12 @@ namespace eval ::Jabber::Login:: {
     set uid 0
     
     # Add all event hooks.
-    ::hooks::register quitAppHook     [list ::UI::SaveWinGeom $wDlgs(jlogin)]
-    ::hooks::register closeWindowHook ::Jabber::Login::CloseHook
+    ::hooks::register quitAppHook     ::Login::QuitAppHook
+    ::hooks::register closeWindowHook ::Login::CloseHook
+    ::hooks::register launchFinalHook ::Login::LaunchHook
 }
 
-# Jabber::Login::Dlg --
+# Login::Dlg --
 #
 #       Log in to a server with an existing user account.
 #
@@ -33,7 +33,7 @@ namespace eval ::Jabber::Login:: {
 # Results:
 #       none
 
-proc ::Jabber::Login::Dlg { } {
+proc ::Login::Dlg { } {
     global  this prefs wDlgs
     
     variable wtoplevel
@@ -87,7 +87,7 @@ proc ::Jabber::Login::Dlg { } {
     grid $frmid.lpop -column 0 -row 0 -sticky e
     grid $wpopup -column 1 -row 0 -sticky e
     
-    ::Jabber::Login::LoadProfiles
+    LoadProfiles
 
     label $frmid.lserv -text "[mc {Jabber server}]:" -font $fontSB -anchor e
     entry $frmid.eserv -width 22    \
@@ -159,11 +159,11 @@ proc ::Jabber::Login::Dlg { } {
     ::UI::SetWindowPosition $w
     wm resizable $w 0 0
     bind $w <Return>  [list $frbot.btok invoke]
-    bind $w <Escape>  [list ::Jabber::Login::DoCancel $w]
-    bind $w <Destroy> [list ::Jabber::Login::DoCancel $w]
+    bind $w <Escape>  [list ::Login::DoCancel $w]
+    bind $w <Destroy> [list ::Login::DoCancel $w]
 }
 
-proc ::Jabber::Login::LoadProfiles { } {
+proc ::Login::LoadProfiles { } {
     global  wDlgs
     variable tmpProfArr
     variable menuVar
@@ -199,7 +199,7 @@ proc ::Jabber::Login::LoadProfiles { } {
     set menuVar $profile
 }
 
-proc ::Jabber::Login::TraceMenuVar {name key op} {
+proc ::Login::TraceMenuVar {name key op} {
     global  prefs
     
     variable profile
@@ -228,17 +228,17 @@ proc ::Jabber::Login::TraceMenuVar {name key op} {
     }
 }
 
-proc ::Jabber::Login::CloseHook {wclose} {
+proc ::Login::CloseHook {wclose} {
     global  wDlgs
     variable finished
     
     if {[string equal $wclose $wDlgs(jlogin)]} {
 	set finished 0
-	::Jabber::Login::Close $wclose
+	Close $wclose
     }
 }
 
-proc ::Jabber::Login::MoreOpts {w} {
+proc ::Login::MoreOpts {w} {
     variable wtri
     variable wtrilab
     variable wfrmore
@@ -249,7 +249,7 @@ proc ::Jabber::Login::MoreOpts {w} {
     bind $wtri <Button-1> [list [namespace current]::LessOpts $w]
 }
 
-proc ::Jabber::Login::LessOpts {w} {
+proc ::Login::LessOpts {w} {
     variable wtri
     variable wtrilab
     variable wfrmore
@@ -260,19 +260,19 @@ proc ::Jabber::Login::LessOpts {w} {
     bind $wtri <Button-1> [list [namespace current]::MoreOpts $w]
 }
 
-proc ::Jabber::Login::DoCancel {w} {
+proc ::Login::DoCancel {w} {
     variable finished
 
     set finished 0
-    ::Jabber::Login::Close $w
+    Close $w
 }
 
-proc ::Jabber::Login::Profiles { } {
+proc ::Login::Profiles { } {
     
     ::Profiles::BuildDialog
 }
 
-proc ::Jabber::Login::Close {w} {
+proc ::Login::Close {w} {
     variable menuVar
     
     # Clean up.
@@ -284,11 +284,11 @@ proc ::Jabber::Login::Close {w} {
     catch {destroy $w}    
 }
 
-# Jabber::Login::DoLogin --
+# Login::DoLogin --
 # 
 #       Starts the login process.
 
-proc ::Jabber::Login::DoLogin {} {
+proc ::Login::DoLogin {} {
     global  prefs
 
     variable wtoplevel
@@ -301,7 +301,7 @@ proc ::Jabber::Login::DoLogin {} {
     upvar ::Jabber::jprefs jprefs
     upvar ::Jabber::jstate jstate
     
-    ::Debug 2 "::Jabber::Login::DoLogin"
+    ::Debug 2 "::Login::DoLogin"
     
     # Kill any pending open states.
     ::Network::KillAll
@@ -312,8 +312,8 @@ proc ::Jabber::Login::DoLogin {} {
     foreach name {server username password} {
 	upvar 0 $name var
 	if {[string length $var] <= 1} {
-	    tk_messageBox -icon error -type ok -message  \
-	      [FormatTextForMessageBox [mc jamessnamemissing $name]]	      
+	    ::UI::MessageBox -icon error -type ok \
+	      -message [mc jamessnamemissing $name]
 	    return
 	}
 	if {$name == "password"} {
@@ -331,112 +331,74 @@ proc ::Jabber::Login::DoLogin {} {
 		}
 	    }
 	} err]} {
-	    tk_messageBox -icon error -type ok -message  \
-	      [FormatTextForMessageBox [mc jamessillegalchar $name $var]]
+	    ::UI::MessageBox -icon error -type ok \
+	      -message [mc jamessillegalchar $name $var]
 	    return
 	}
     }    
     set finished 1
-    ::Jabber::Login::Close $wtoplevel
+    Close $wtoplevel
     
     set opts {}
-    if {$moreOpts(httpproxy)} {
-	lappend opts -httpproxy 1
+    foreach {key value} [array get moreOpts] {
+	lappend opts -$key $value
     }
-    eval {::Jabber::Login::Connect $server [namespace current]::DoLoginCB \
-      -ssl $moreOpts(ssl) -ip $moreOpts(ip) -port $moreOpts(port)} $opts
+    # ssl vs. tls naming conflict.
+    if {$moreOpts(ssl)} {
+	lappend opts -tls 1
+    }
+    eval {HighLogin $server $username $resource $password \
+      [namespace current]::LoginCallback} $opts
 }
 
-proc ::Jabber::Login::DoLoginCB {status msg} {
-    variable server
-    variable moreOpts
-
-    ::Debug 2 "::Jabber::Login::DoLoginCB status=$status"
+proc ::Login::LoginCallback {token status {errmsg ""}} {
+    variable $token
+    upvar 0 $token state
     
-    switch $status {
-	error {
-	    tk_messageBox -icon error -type ok -message [FormatTextForMessageBox \
-	      [mc jamessnosocket $server $msg]]
+    ::Debug 2 "::Login::LoginCallback"
+    
+    ShowAnyMessageBox $token $status $errmsg
+}
+
+#       Show message box if necessary.
+
+proc ::Login::ShowAnyMessageBox {token status {errmsg ""}} {
+    variable $token
+    upvar 0 $token state
+    
+    ::Debug 2 "::Login::DoLoginHighCB status=$status errmsg=$errmsg"
+    
+    set str ""
+    
+    switch -- $status {
+	ok {
+	    # empty
+	}
+	connect-failed {
+	    set str [mc jamessnosocket $state(server) $errmsg]
 	}
 	timeout {
-	    tk_messageBox -icon error -type ok -message [FormatTextForMessageBox \
-	      [mc jamesstimeoutserver $server]]
+	    set str [mc jamesstimeoutserver $state(server)]
+	}
+	authfail {
+	    set str $errmsg
+	}
+	starttls-nofeature {
+	    set str [mc jamessstarttls-nofeature $state(server)]
+	}
+	startls-failure {
+	    set str [mc jamessstarttls-failure $state(server)]
 	}
 	default {
-	    # Go ahead...
-	    set opts {}
-	    if {$moreOpts(sasl)} {
-		lappend opts -sasl 1
-	    }
-	    if {[catch {
-		eval {::Jabber::Login::InitStream $server \
-		  [namespace current]::DoAuthorize} $opts
-	    } err]} {
-		tk_messageBox -icon error -title [mc {Open Failed}] \
-		  -type ok -message [FormatTextForMessageBox $err]
-	    }
+	    set str $errmsg
 	}
     }
+    if {$str != ""} {
+	::UI::MessageBox -icon error -type ok -message $str
+    }
 }
 
-proc ::Jabber::Login::DoAuthorize {args} {
-    variable server
-    variable username
-    variable password
-    variable resource
-    variable moreOpts
-    
-    ::Debug 2 "::Jabber::Login::DoAuthorize args=$args"
-    array set argsArr $args
-    if {![info exists argsArr(id)]} {
-	tk_messageBox -icon error -type ok -message \
-	  "no id for digest in receiving <stream>"
-    } else {
-	if {$resource == ""} {
-	    set resource "coccinella"
-	}
-	
-	# If we are trying to use sasl indicated by version='1.0' we must also
-	# be sure to receive a version attribute larger or equal to 1.0.
-	set trysasl $moreOpts(sasl)
-	if {$moreOpts(sasl)} {
-	    if {[info exists argsArr(version)]} {
-		if {[package vcompare $argsArr(version) 1.0] == -1} {
-		    set moreOpts(sasl) 0
-		}
-	    } else {
-		set moreOpts(sasl) 0
-	    }
-	}
-	if {$trysasl && !$moreOpts(sasl)} {
-	    ::Jabber::AddErrorLog $server  \
-	      "SASL authentization failed since server does not support version=1.0"
-	}
-	::Jabber::Login::Authorize $server $username $resource $password \
-	  [namespace current]::Finish -streamid $argsArr(id)  \
-	  -digest $moreOpts(digest) -sasl $moreOpts(sasl)
-    }
-}
-
-proc ::Jabber::Login::Finish {type msg} {
-    variable profile
-    variable moreOpts
-    
-    ::Debug 2 "::Jabber::Login::Finish type=$type, msg=$msg"
-
-    if {[string equal $type "error"]} {
-	tk_messageBox -icon error -type ok -title [mc Error]  \
-	  -message [FormatTextForMessageBox $msg]
-    } else {
-	::Profiles::SetSelectedName $profile
-	
-	# Login was succesful, set presence.
-	::Jabber::Login::SetStatus -priority $moreOpts(priority)  \
-	  -invisible $moreOpts(invisible)
-    }
-}
-
-proc ::Jabber::Login::SetStatus {args} {
+proc ::Login::SetStatus {args} {
     upvar ::Jabber::jstate jstate
     
     array set argsArr {
@@ -450,27 +412,308 @@ proc ::Jabber::Login::SetStatus {args} {
     }
     
     if {$argsArr(-invisible)} {
-	set jstate(status) "invisible"
 	eval {::Jabber::SetStatus invisible} $presArgs
     } else {
-	set jstate(status) "available"
 	eval {::Jabber::SetStatus available -notype 1} $presArgs
     }    
 }
 
+# Login::LaunchHook --
+# 
+#       Method to automatically login after launch.
+
+proc ::Login::LaunchHook { } {
+    upvar ::Jabber::jprefs jprefs
+    
+    if {!$jprefs(autoLogin)} {
+	return ""
+    }
+    
+    # Use our selected profile.
+    set profname [::Profiles::GetSelectedName]
+    set password [::Profiles::Get $profname password]
+    set ans "ok"
+    if {$password == ""} {
+	set ans [::UI::MegaDlgMsgAndEntry  \
+	  [mc {Password}] [mc enterpassword $state(jid)] "[mc Password]:" \
+	  password [mc Cancel] [mc OK] -show {*}]
+    }
+    if {$ans == "ok"} {
+	set domain [::Profiles::Get $profname domain]
+	set node   [::Profiles::Get $profname node]
+	set opts   [::Profiles::Get $profname options]
+	array set optsArr $opts
+	if {[info exists optsArr(-resource)] && ($optsArr(-resource) != "")} {
+	    set res $optsArr(-resource)
+	} else {
+	    set res "coccinella"
+	}
+	eval {::Login::HighLogin $domain $node $res $password \
+	  [namespace current]::AutoLoginCB} $opts
+    }
+}
+
+proc ::Login::QuitAppHook { } {
+    global  wDlgs
+    
+    ::UI::SaveWinGeom $wDlgs(jlogin)    
+}
+
+proc ::Login::AutoLoginCB {logtoken status {errmsg ""}} {
+
+    ::Login::ShowAnyMessageBox $logtoken $status $errmsg
+}
+
+#-------------------------------------------------------------------------------
+
+# Initialize the complete login processs and receive callback when finished.
+# Sort of high level call at application level. Handles all UI except
+# any message boxes.
+
+# Login::HighLogin --
+# 
+#       Initializes the login procedure. Callback when finished with status.
+#       
+# Arguments:
+#       server
+#       username
+#       resource
+#       password
+#       cmd         callback command
+#       args:
+#           -digest  0|1
+#           -httpproxy
+#           -invisible  0|1
+#           -ip
+#           -priority
+#           -sasl
+#           -ssl        0|1 (synonym for -tls)
+#           -tls
+#       
+# Results:
+#       Callback initiated.
+
+proc ::Login::HighLogin {server username resource password cmd args} {
+    global  prefs
+    variable uid
+    upvar ::Jabber::jprefs jprefs
+    upvar ::Jabber::jstate jstate
+    
+    ::Debug 2 "::Login::HighLogin args=$args"
+    
+    array set argsArr {
+	-digest         1
+	-httpproxy      0
+	-invisible      0
+	-ip             ""
+	-priority       0
+	-sasl           0
+	-ssl            0
+	-tls            0
+    }
+    set argsArr(-port) $jprefs(port)
+    array set argsArr $args
+    if {$resource == ""} {
+	set resource "coccinella"
+    }
+
+    # Initialize the state variable, an array, that keeps is the storage.
+    
+    set token [namespace current]::high[incr uid]
+    variable $token
+    upvar 0 $token state
+
+    set state(server)     $server
+    set state(username)   $username
+    set state(resource)   $resource
+    set state(password)   $password
+    set state(cmd)        $cmd
+    foreach {key value} [array get argsArr] {
+	set state($key) $value
+    }
+    
+    # -ssl synonym for -tls.
+    if {$state(-ssl)} {
+	set state(-tls) 1
+    }
+    
+    # The "old" jabber protocol uses a designated port number for ssl
+    # connections. 
+    # In xmpp tls negotiating is made in stream using the standard port.
+    set opts {}
+    if {!$state(-sasl) && $state(-tls)} {
+	lappend opts -tls 1
+    }
+    if {$state(-httpproxy)} {
+	lappend opts -httpproxy 1
+    }
+    if {$state(-ip) != ""} {
+	lappend opts -ip $argsArr(-ip)
+    }
+    
+    # Make a network connection.
+    eval {Connect $server [list [namespace current]::HighConnectCB $token] \
+      -port $state(-port)} $opts
+}
+
+proc ::Login::HighConnectCB {token status msg} {
+    variable $token
+    upvar 0 $token state
+    
+    ::Debug 2 "::Login::HighConnectCB status=$status"
+    
+    switch $status {
+	error {
+	    HighFinish $token connect-failed $msg
+	}
+	timeout {
+	    HighFinish $token timeout $msg
+	}
+	default {
+	    set opts {}
+	    if {$state(-sasl)} {
+		lappend opts -sasl 1
+	    }
+	    if {[catch {
+		eval {InitStream $state(server) \
+		  [list [namespace current]::HighInitStreamCB $token]} $opts
+	    } err]} {
+		HighFinish $token connect-failed $err
+	    }
+	}
+    }
+}
+
+proc ::Login::HighInitStreamCB {token args} {
+    variable $token
+    upvar 0 $token state
+    upvar ::Jabber::jstate jstate
+
+    ::Debug 2 "::Login::HighInitStreamCB"
+    
+    array set argsArr $args
+
+    if {![info exists argsArr(id)]} {
+	HighFinish $token missingid "no id for digest in receiving <stream>"
+    } else {
+
+	# If we are trying to use sasl indicated by version='1.0' we must also
+	# be sure to receive a version attribute larger or equal to 1.0.
+	set trysasl $state(-sasl)
+	if {$state(-sasl)} {
+	    if {[info exists argsArr(version)]} {
+		if {[package vcompare $argsArr(version) 1.0] == -1} {
+		    set state(-sasl) 0
+		}
+	    } else {
+		set state(-sasl) 0
+	    }
+	}
+	if {$trysasl && !$state(-sasl)} {
+	    ::Jabber::AddErrorLog $state(server)  \
+	      "SASL authentization failed since server does not support version=1.0"
+	}
+	set starttls 0
+	if {$state(-sasl) && $state(-tls)} {
+	    set starttls 1
+	}
+	
+	# We cannot verify that server supports tls since we have not yet
+	# received the 'features' element. Done inside jlib.
+		
+	# We either start tls or authorize.
+	if {$starttls} {
+	    StartTls [list [namespace current]::HighStartTlsCB $token]
+	} else {
+	    Authorize $state(server) $state(username) $state(resource) \
+	      $state(password) \
+	      [list [namespace current]::HighAuthorizeCB $token] \
+	      -streamid $argsArr(id) -digest $state(-digest) -sasl $state(-sasl)
+	}    
+    }    
+}
+
+proc ::Login::HighStartTlsCB {token type args} {
+    variable $token
+    upvar 0 $token state
+    upvar ::Jabber::jstate jstate
+
+    ::Debug 2 "::Login::HighStartTlsCB type=$type args=$args"
+    
+    if {[string equal $type "error"]} {
+	foreach {errcode errmsg} [lindex $args 0] break
+	HighFinish $token $errcode $errmsg
+    } else {
+	set id [$jstate(jlib) getstreamattr id]
+	if {$id == ""} {
+	    HighFinish $token missingid \
+	      "no id for digest in receiving <stream>"
+	} else {
+	    Authorize $state(server) $state(username) $state(resource) \
+	      $state(password) \
+	      [list [namespace current]::HighAuthorizeCB $token] \
+	      -streamid $id -digest $state(-digest) -sasl $state(-sasl)
+	}
+    }
+}
+
+proc ::Login::HighAuthorizeCB {token type msg} {
+    variable $token
+    upvar 0 $token state
+    upvar ::Jabber::jstate jstate
+    
+    ::Debug 2 "::Login::HighAuthorizeCB type=$type"
+
+    switch -- $type {
+	error {
+	    HighFinish $token authfail $msg
+	}
+	default {
+
+	    # Login was succesful, set presence.
+	    set opts {}
+	    if {$state(-priority) != 0} {
+		lappend opts -priority $state(-priority)
+	    }
+	    if {$state(-invisible)} {
+		eval {::Jabber::SetStatus invisible} $opts
+	    } else {
+		eval {::Jabber::SetStatus available -notype 1} $opts
+	    }    
+	    HighFinish $token
+	}
+    }
+}
+
+proc ::Login::HighFinish {token {err ""} {msg ""}} {
+    variable $token
+    upvar 0 $token state
+   
+    ::Jabber::UI::SetStatusMessage ""
+    ::Jabber::UI::StartStopAnimatedWave 0
+    if {$err == ""} {
+	uplevel #0 $state(cmd) $token ok
+    } else {
+	::Jabber::UI::FixUIWhen "disconnect"
+	uplevel #0 $state(cmd) [list $token $err $msg]
+    }
+    unset state
+}
+
 #-------------------------------------------------------------------------------
 
 # A few functions to handle each stage in the login process as a separate
 # component:
 #       o connect socket
 #       o initialize the stream
+#       o start tls
 #       o authorize
 #       
 # They get and set jstate and other state variables. User interface elements
 # are set, but no message boxes are shown. That is up to the caller.
 
 
-# Jabber::Login::Connect --
+# Login::Connect --
 #
 #       Initiates a login to a server with an existing user account.
 #
@@ -479,22 +722,23 @@ proc ::Jabber::Login::SetStatus {args} {
 #       cmd         callback command when socket connected
 #       args  -ip
 #             -port
-#             -ssl 0|1
+#             -tls 0|1
 #             -httpproxy 0|1
 #       
 # Results:
 #       Callback scheduled.
 
-proc ::Jabber::Login::Connect {server cmd args} {
+proc ::Login::Connect {server cmd args} {
     global  prefs
 
     upvar ::Jabber::jprefs jprefs
     upvar ::Jabber::jstate jstate
     
-    ::Debug 2 "::Jabber::Login::Connect args=$args"
+    ::Debug 2 "::Login::Connect args=$args"
+    
     array set argsArr {
 	-ip         ""
-	-ssl        0
+	-tls        0
 	-httpproxy  0
     }
     array set argsArr $args
@@ -508,7 +752,7 @@ proc ::Jabber::Login::Connect {server cmd args} {
     update idletasks
 
     # Async socket open with callback.
-    if {$argsArr(-ssl)} {
+    if {$argsArr(-tls)} {
 	set port $jprefs(sslport)
     } elseif {[info exists argsArr(-port)]} {
 	set port $argsArr(-port)
@@ -521,10 +765,14 @@ proc ::Jabber::Login::Connect {server cmd args} {
 	set host $argsArr(-ip)
     }
     
+    # The "old" jabber protocol uses a designated port number for ssl
+    # connections. 
+    # In xmpp tls negotiating is made in stream using the standard port.
+    
     # Open socket unless we are using a http proxy.
     if {!$argsArr(-httpproxy)} {
 	::Network::Open $host $port [list [namespace current]::ConnectCB $cmd] \
-	  -timeout $prefs(timeoutSecs) -tls $argsArr(-ssl)
+	  -timeout $prefs(timeoutSecs) -tls $argsArr(-tls)
     } else {
 	
 	# Perhaps it gives a better structure to have this elsewhere?
@@ -549,24 +797,24 @@ proc ::Jabber::Login::Connect {server cmd args} {
     }
 }
 
-proc ::Jabber::Login::HttpProxyCmd {status msg} {
+proc ::Login::HttpProxyCmd {status msg} {
     
-    ::Debug 2 "::Jabber::Login::HttpProxyCmd status=$status, msg=$msg"
+    ::Debug 2 "::Login::HttpProxyCmd status=$status, msg=$msg"
     
     switch -- $status {
 	ok {
-	    
+	    # only errors are handled via callback
 	}
 	default {
 	    ::Jabber::DoCloseClientConnection
-	    tk_messageBox -title [mc Error] -icon error \
+	    ::UI::MessageBox -title [mc Error] -icon error \
 	      -message "The HTTP jabber service replied with a status\
 	      $status and message: $msg"
 	}
     }
 }
 
-# Jabber::Login::ConnectCB --
+# Login::ConnectCB --
 #
 #       Callback when socket has been opened. Logins.
 #       
@@ -577,10 +825,10 @@ proc ::Jabber::Login::HttpProxyCmd {status msg} {
 # Results:
 #       Callback initiated.
 
-proc ::Jabber::Login::ConnectCB {cmd sock ip port status {msg {}}} {    
+proc ::Login::ConnectCB {cmd sock ip port status {msg {}}} {    
     upvar ::Jabber::jstate jstate
     
-    ::Debug 2 "::Jabber::Login::ConnectCB"
+    ::Debug 2 "::Login::ConnectCB"
     
     switch $status {
 	error - timeout {
@@ -590,14 +838,13 @@ proc ::Jabber::Login::ConnectCB {cmd sock ip port status {msg {}}} {
 	}
 	default {
 	    set jstate(sock) $sock
-	    
 	    $jstate(jlib) setsockettransport $jstate(sock)
 	}
     }    
     uplevel #0 $cmd [list $status $msg]
 }
 
-# Jabber::Login::InitStream --
+# Login::InitStream --
 # 
 #       Sends the init stream xml command. When received servers stream,
 #       invokes the cmd callback.
@@ -609,7 +856,7 @@ proc ::Jabber::Login::ConnectCB {cmd sock ip port status {msg {}}} {
 # Results:
 #       Callback initiated.
 
-proc ::Jabber::Login::InitStream {server cmd args} {
+proc ::Login::InitStream {server cmd args} {
     upvar ::Jabber::jstate jstate
     
     ::Jabber::UI::SetStatusMessage [mc jawaitxml $server]
@@ -634,13 +881,34 @@ proc ::Jabber::Login::InitStream {server cmd args} {
     }
 }
 
-proc ::Jabber::Login::InitStreamCB {cmd jlibName args} {
+proc ::Login::InitStreamCB {cmd jlibName args} {
     
     # One of args shall be -id hexnumber
     uplevel #0 $cmd $args
 }
 
-# Jabber::Login::Authorize --
+# Login::StartTls --
+# 
+# 
+
+proc ::Login::StartTls {cmd} {
+    upvar ::Jabber::jstate jstate
+    
+    ::Jabber::UI::SetStatusMessage [mc jatlsnegot]
+    $jstate(jlib) starttls [list [namespace current]::StartTlsCB $cmd]    
+}
+
+proc ::Login::StartTlsCB {cmd jlibName type args} {
+    upvar ::Jabber::jstate jstate
+
+    if {[string equal $type "error"]} {	
+	::Jabber::UI::FixUIWhen "disconnect"
+	after idle $jstate(jlib) closestream
+    }    
+    uplevel #0 $cmd $type $args
+}
+
+# Login::Authorize --
 # 
 #       A fairly general method for authentication. Handles UI stuff, but
 #       does not show any message boxes.
@@ -659,11 +927,12 @@ proc ::Jabber::Login::InitStreamCB {cmd jlibName args} {
 # Results:
 #       Callback initiated.
 
-proc ::Jabber::Login::Authorize {server username resource password cmd args} {
+proc ::Login::Authorize {server username resource password cmd args} {
     variable uid
     upvar ::Jabber::jstate jstate
     
-    ::Debug 2 "::Jabber::Login::Authorize"
+    ::Debug 2 "::Login::Authorize"
+    
     array set argsArr {
 	-digest     1
 	-streamid   ""
@@ -695,12 +964,14 @@ proc ::Jabber::Login::Authorize {server username resource password cmd args} {
 	$jstate(jlib) send_auth $username $resource   \
 	  [list [namespace current]::AuthorizeCB $token] -digest $digestedPw
     } else {
-	$jstate(jlib) auth_sasl $username $resource $password  \
-	  [list [namespace current]::AuthorizeCB $token] 
+	
+	# Plain password authentization.
+	$jstate(jlib) send_auth $username $resource   \
+	  [list [namespace current]::AuthorizeCB $token] -password $password
     }
 }
 
-proc ::Jabber::Login::AuthorizeCB {token jlibName type theQuery} {
+proc ::Login::AuthorizeCB {token jlibName type theQuery} {
     global  this
 
     variable $token
@@ -708,7 +979,7 @@ proc ::Jabber::Login::AuthorizeCB {token jlibName type theQuery} {
     upvar ::Jabber::jstate jstate
     upvar ::Jabber::jserver jserver
 
-    ::Debug 2 "::Jabber::Login::AuthorizeCB type=$type, theQuery='$theQuery'"
+    ::Debug 2 "::Login::AuthorizeCB type=$type, theQuery='$theQuery'"
     
     set server   $state(server)
     set username $state(username)
@@ -720,8 +991,7 @@ proc ::Jabber::Login::AuthorizeCB {token jlibName type theQuery} {
     ::Jabber::UI::StartStopAnimatedWave 0
     
     if {[string equal $type "error"]} {	
-	set errcode [lindex $theQuery 0]
-	set errmsg [lindex $theQuery 1]
+	foreach {errcode errmsg} $theQuery break
 	::Jabber::UI::SetStatusMessage [mc jaerrlogin $server $errmsg]
 	::Jabber::UI::FixUIWhen "disconnect"
 	if {$errcode == 409} {
