@@ -4,7 +4,7 @@
 #      
 #  Copyright (c) 2003-2004  Mats Bengtsson
 #  
-# $Id: Profiles.tcl,v 1.31 2004/09/28 13:50:18 matben Exp $
+# $Id: Profiles.tcl,v 1.36 2004/12/02 08:22:34 matben Exp $
 
 package provide Profiles 1.0
 
@@ -66,7 +66,7 @@ proc ::Profiles::Set {name server username password args} {
     ::Debug 2 "::Profiles::Set: name=$name, s=$server, u=$username, p=$password, '$args'"
 
     array set profArr $profiles
-    set allNames [::Profiles::GetAllNames]
+    set allNames [GetAllNames]
     
     # Create a new unique profile name.
     if {[string length $name] == 0} {
@@ -88,7 +88,7 @@ proc ::Profiles::Set {name server username password args} {
     set selected $name
     
     # Keep them sorted.
-    ::Profiles::SortProfileList
+    SortProfileList
     return ""
 }
 
@@ -201,7 +201,7 @@ proc ::Profiles::GetProfile {name} {
 proc ::Profiles::GetSelectedName { } {
     variable selected
  
-    set all [::Profiles::GetAllNames]
+    set all [GetAllNames]
     if {[lsearch -exact $all $selected] < 0} {
 	return [lindex $all 0]
     } else {
@@ -238,7 +238,7 @@ proc ::Profiles::SortProfileList { } {
 
     set tmp {}
     array set profArr $profiles
-    foreach name [::Profiles::GetAllNames] {
+    foreach name [GetAllNames] {
 	lappend tmp $name $profArr($name)
     }
     set profiles $tmp
@@ -254,7 +254,7 @@ proc ::Profiles::ImportIfNecessary { } {
 	# Use ad hoc method to figure out if want to import profiles.
 	if {[string equal [lindex $profiles 1 1] "myUsername"] && \
 	  ([llength $profiles] == 2)} {
-	    ::Profiles::ImportFromJserver
+	    ImportFromJserver
 	}
     }
 }
@@ -287,7 +287,7 @@ proc ::Profiles::BuildHook {wtree nbframe} {
       -text [mc {User Profiles}]
 
     set wpage [$nbframe page {User Profiles}]    
-    ::Profiles::BuildPage $wpage
+    BuildPage $wpage
 }
 
 proc ::Profiles::BuildPage {page} {
@@ -327,7 +327,7 @@ proc ::Profiles::BuildPage {page} {
     pack [frame $pui] -side left  
     
     # Make temp array for servers.
-    ::Profiles::MakeTmpProfArr
+    MakeTmpProfArr
     set tmpSelected $selected
 		
     # Init textvariables.
@@ -336,7 +336,7 @@ proc ::Profiles::BuildPage {page} {
     set username $tmpProfArr($profile,username)
     set password $tmpProfArr($profile,password)
     
-    set allNames [::Profiles::GetAllNames]
+    set allNames [GetAllNames]
 
     # Option menu for the servers.
     label $pui.lpop -text "[mc Profile]:" -anchor e
@@ -403,11 +403,11 @@ proc ::Profiles::BuildPage {page} {
 
     # Tabbed notebook for more options.
     set wtabnb $popt.nb
-    ::Profiles::NotebookOptionWidget $wtabnb $token
+    NotebookOptionWidget $wtabnb $token
     pack $wtabnb -fill x
     
     # The actual prefs state for the current profile must be set.
-    ::Profiles::SetCurrentFromTmp $tmpSelected
+    SetCurrentFromTmp $tmpSelected
 
     # This allows us to clean up some things when we go away.
     bind $lfr <Destroy> [list [namespace current]::DestroyHandler]
@@ -497,9 +497,10 @@ proc ::Profiles::NotebookOptionWidget {w token} {
 	grid $pageproxy.lpoll $pageproxy.spoll -sticky e
 	grid $pageproxy.set   -   -sticky w
     }
+    #bind [winfo toplevel $w] <Control-Key-t> [list $w nextpage]
     
     # Set defaults.
-    ::Profiles::NotebookSetDefaults $token
+    NotebookSetDefaults $token
 
     # Let components ad their own stuff here.
     ::hooks::run profileBuildTabNotebook $w $token
@@ -525,7 +526,7 @@ proc ::Profiles::NotebookSetDefaults {token} {
     Debug 2 "::Profiles::NotebookSetDefaults"
 
     if {!$initedDefaultOptions} {
-	::Profiles::InitDefaultOptions
+	InitDefaultOptions
     }
     array set state [array get defaultOptionsArr]
 }
@@ -587,7 +588,7 @@ proc ::Profiles::TraceProfile {name key op} {
     variable profile
 
     Debug 4 "TraceProfile name=$name; set name=[set $name]"
-    ::Profiles::SetCmd [set $name]
+    SetCmd [set $name]
 }
 
 # Profiles::SetCmd --
@@ -609,19 +610,19 @@ proc ::Profiles::SetCmd {profName} {
     if {$previousExists} {
 	
 	# Check if there are any empty fields.
-	if {![::Profiles::VerifyNonEmpty]} {
+	if {![VerifyNonEmpty]} {
 	    set profile $tmpSelected
 	    Debug 2 "***::Profiles::VerifyNonEmpty: set profile $tmpSelected"
 	    return
 	}
 	
 	# Save previous state in tmp before setting the new one.
-	::Profiles::SaveCurrentToTmp $tmpSelected
+	SaveCurrentToTmp $tmpSelected
     }
     
     # In case this is a new profile.
     if {[info exists tmpProfArr($profName,server)]} {
-	::Profiles::SetCurrentFromTmp $profName
+	SetCurrentFromTmp $profName
     }
 }
 
@@ -640,19 +641,19 @@ proc ::Profiles::SetCurrentFromTmp {profName} {
     set server   $tmpProfArr($profName,server)
     set username $tmpProfArr($profName,username)
     set password $tmpProfArr($profName,password)
-    ::Profiles::NotebookSetDefaults [namespace current]::moreOpts
+    NotebookSetDefaults [namespace current]::moreOpts
     
     foreach {key value} [array get tmpProfArr $profName,-*] {
 	set optname [string map [list $profName,- ""] $key]
+	Debug 4 "\t key=$key, value=$value, optname=$optname"
 	
 	# The 'resource' is a bit special...
 	if {$optname == "resource"} {
-	    continue
+	    set resource $value
+	} else {
+	    set moreOpts($optname) $value
 	}
-	Debug 4 "\t key=$key, value=$value, optname=$optname"
-	set moreOpts($optname) $value
     }
-    set resource $tmpProfArr($profName,-resource)
     set tmpSelected $profName  
 }
 
@@ -703,9 +704,8 @@ proc ::Profiles::VerifyNonEmpty { } {
     
     # Check that necessary entries are non-empty, at least.
     if {($server == "") || ($username == "")} {
-	tk_messageBox -type ok -icon error -parent [winfo toplevel $wpage] \
-	  -title [mc Error] -message [FormatTextForMessageBox \
-	  [mc messfillserveruser]]
+	::UI::MessageBox -type ok -icon error -parent [winfo toplevel $wpage] \
+	  -title [mc Error] -message [mc messfillserveruser]
 	set ans 0
     }
     return $ans
@@ -715,7 +715,7 @@ proc ::Profiles::MakeUniqueProfileName {name} {
     variable server
     
     # Create a unique profile name if not given.
-    set allNames [::Profiles::GetAllTmpNames]
+    set allNames [GetAllTmpNames]
     if {$name == ""} {
 	set name $server
     }
@@ -759,16 +759,14 @@ proc ::Profiles::NewCmd { } {
     set newProfile ""
     
     # First get a unique profile name.
-    set ans [::UI::MegaDlgMsgAndEntry \
-      [mc Profile] [mc prefprofname] \
-      "[mc {Profile Name}]:" newProfile \
-      [mc Cancel] [mc OK]]
+    set ans [::UI::MegaDlgMsgAndEntry [mc Profile] [mc prefprofname] \
+      "[mc {Profile Name}]:" newProfile [mc Cancel] [mc OK]]
     if {$ans == "cancel"} {
 	return
     }
     Debug 2 "::Profiles::NewCmd tmpSelected=$tmpSelected, newProfile=$newProfile"
 
-    set uniqueName [::Profiles::MakeUniqueProfileName $newProfile]
+    set uniqueName [MakeUniqueProfileName $newProfile]
     $wmenu add radiobutton -label $uniqueName  \
       -variable [namespace current]::profile
 
@@ -778,7 +776,7 @@ proc ::Profiles::NewCmd { } {
     set username  ""
     set password  ""
     set resource  ""
-    ::Profiles::NotebookSetDefaults [namespace current]::moreOpts
+    NotebookSetDefaults [namespace current]::moreOpts
     
     # Must do this for it to be automatically saved.
     set tmpProfArr($profile,name) $profile
@@ -799,10 +797,10 @@ proc ::Profiles::DeleteCmd { } {
     
     # The present state may be something that has not been stored yet.
     if {[info exists tmpProfArr($profile,server)]} {
-	set ans [tk_messageBox -title [mc Warning]  \
+	set ans [::UI::MessageBox -title [mc Warning]  \
 	  -type yesno -icon warning -default yes  \
 	  -parent [winfo toplevel $wpage] \
-	  -message [FormatTextForMessageBox [mc messremoveprofile]]]
+	  -message [mc messremoveprofile]]
     }
     if {$ans == "yes"} {
 	set ind [$wmenu index $profile]
@@ -810,11 +808,11 @@ proc ::Profiles::DeleteCmd { } {
 	    $wmenu delete $ind
 	}
 	array unset tmpProfArr "$profile,*"
-	set allNames [::Profiles::GetAllTmpNames]
+	set allNames [GetAllTmpNames]
 	
 	# Set selection to first.
 	set profile [lindex $allNames 0]
-	::Profiles::SetCmd $profile
+	SetCmd $profile
     }
 }
 
@@ -833,11 +831,11 @@ proc ::Profiles::SaveHook { } {
     variable selected
     variable tmpSelected
 
-    set profiles [::Profiles::GetTmpProfiles]
+    set profiles [GetTmpProfiles]
     set selected $tmpSelected
     
     # Update the Login dialog if any.
-    ::Jabber::Login::LoadProfiles
+    ::Login::LoadProfiles
 }
 
 proc ::Profiles::GetTmpProfiles { } {
@@ -847,10 +845,10 @@ proc ::Profiles::GetTmpProfiles { } {
     Debug 2 "::Profiles::GetTmpProfiles"
     
     # Get present dialog state into tmp array first.
-    ::Profiles::SaveCurrentToTmp $profile
+    SaveCurrentToTmp $profile
     
     set tmpProfiles {}
-    foreach name [::Profiles::GetAllTmpNames] {
+    foreach name [GetAllTmpNames] {
 	set plist [list $tmpProfArr($name,server) $tmpProfArr($name,username) \
 	  $tmpProfArr($name,password)]
 	
@@ -884,7 +882,7 @@ proc ::Profiles::CancelHook { } {
 	::Preferences::HasChanged
 	return
     }
-    set tmpProfiles [::Profiles::GetTmpProfiles]
+    set tmpProfiles [GetTmpProfiles]
     if {![string equal $profiles $tmpProfiles]} {
 	::Preferences::HasChanged
 	return
@@ -896,10 +894,10 @@ proc ::Profiles::UserDefaultsHook { } {
     variable tmpSelected
     variable profile
     
-    ::Profiles::MakeTmpProfArr
+    MakeTmpProfArr
     set tmpSelected $selected
     set profile $selected
-    ::Profiles::SetCmd $selected
+    SetCmd $selected
 }
 
 # Standalone dialog --
@@ -922,7 +920,7 @@ proc ::Profiles::BuildDialog { } {
 
     set wpage $w.frall.page
     pack [frame $wpage] -padx 4 -pady 4
-    ::Profiles::BuildPage $wpage
+    BuildPage $wpage
     
     # Button part.
     set frbot [frame $w.frall.frbot -borderwidth 0]
@@ -942,14 +940,14 @@ proc ::Profiles::CloseDlgHook {wclose} {
     global  wDlgs
 
     if {[string equal $wclose $wDlgs(jprofiles)]} {
-	::Profiles::CancelDlg $wclose
+	CancelDlg $wclose
     }
 }
 
 proc ::Profiles::SaveDlg {w} {
     
     ::UI::SaveWinGeom $w
-    ::Profiles::SaveHook
+    SaveHook
     destroy $w
 }
 
