@@ -7,9 +7,9 @@
 #  
 #  See the README file for license, bugs etc.
 #  
-# $Id: Preferences.tcl,v 1.64 2004/10/04 09:22:19 matben Exp $
+# $Id: Preferences.tcl,v 1.71 2004/12/20 15:16:46 matben Exp $
  
-package require notebook
+package require mnotebook
 package require tree
 package require tablelist
 package require combobox
@@ -17,7 +17,7 @@ package require combobox
 package provide Preferences 1.0
 
 namespace eval ::Preferences:: {
-    global  wDlgs this
+    global  this
     
     # Variable to be used in tkwait.
     variable finished
@@ -26,7 +26,7 @@ namespace eval ::Preferences:: {
     variable lastPage {}
 
     # Add all event hooks.
-    ::hooks::register quitAppHook     [list ::UI::SaveWinGeom $wDlgs(prefs)]
+    ::hooks::register quitAppHook     ::Preferences::QuitAppHook
     ::hooks::register closeWindowHook ::Preferences::CloseHook
 
     variable xpadbt
@@ -69,6 +69,12 @@ namespace eval ::Preferences:: {
     option add *Preferences.yPadBig             $ypadbig      widgetDefault
 }
 
+proc ::Preferences::QuitAppHook { } {
+    global  wDlgs
+
+    ::UI::SaveWinGeom $wDlgs(prefs)
+}
+
 proc ::Preferences::Build {args} {
     global  this prefs wDlgs
     
@@ -120,7 +126,7 @@ proc ::Preferences::Build {args} {
     pack [label $frtree.la -text [mc {Settings Panels}]  \
       -font $fontSB -relief raised -width 24 -bd 1 -bg #bdbdbd] -side top -fill x
     set wtree $frtree.t
-    ::tree::tree $wtree -width 100 -height 300 \
+    ::tree::tree $wtree -width 100 -height 300 -indention 0 \
       -yscrollcommand [list ::UI::ScrollSet $frtree.sby \
       [list pack $frtree.sby -side right -fill y]]  \
       -selectcommand ::Preferences::SelectCmd   \
@@ -136,10 +142,9 @@ proc ::Preferences::Build {args} {
     if {!$prefs(stripJabber)} {
 	$wtree newitem {Jabber}
     }
-    $wtree newitem {Whiteboard} -text [mc Whiteboard]
     
     # The notebook and its pages.
-    set nbframe [notebook::notebook $w.frall.fr.nb -borderwidth 1 -relief sunken]
+    set nbframe [::mnotebook::mnotebook $w.frall.fr.nb -borderwidth 1 -relief sunken]
     pack $nbframe -expand 1 -fill both -padx 4 -pady 4
     
     # Make the notebook pages.
@@ -205,9 +210,8 @@ proc ::Preferences::ResetToFactoryDefaults {maxPriorityNum} {
     Debug 2 "::Preferences::ResetToFactoryDefaults maxPriorityNum=$maxPriorityNum"
     
     # Warn first.
-    set ans [tk_messageBox -title [mc Warning] -type yesno -icon warning \
-      -message [FormatTextForMessageBox [mc messfactorydefaults]] \
-      -default no]
+    set ans [::UI::MessageBox -title [mc Warning] -type yesno -icon warning \
+      -message [mc messfactorydefaults] -default no]
     if {$ans == "no"} {
 	return
     }
@@ -686,34 +690,36 @@ proc ::Preferences::Proxies::BuildPage {page} {
     grid $pcnat.cb -sticky w -pady $ypad
     grid $pcnat.eip -sticky ew -pady $ypad
     
-    set pca $page.fr
-    labelframe $pca -text {Http Proxy} -padx 12 -pady 4
-    pack $pca -side top -anchor w -padx 8 -pady 4
-
-    label $pca.msg -wraplength 300 -justify left \
-      -text "Usage of the Http proxy is determined\
-      by each profile settings. File transfers wont work if you use Http proxy!\
-      NOT YET WORKING!"
-    
-    label $pca.lserv -text [mc {Proxy Server}]:
-    entry $pca.eserv -textvariable [namespace current]::tmpPrefs(httpproxyserver)
-    label $pca.lport -text [mc {Proxy Port}]:
-    entry $pca.eport -textvariable [namespace current]::tmpPrefs(httpproxyport)
-    checkbutton $pca.auth -text " [mc {Use proxy authorization}]" \
-      -textvariable [namespace current]::tmpPrefs(httpproxyauth)
-    label $pca.luser -text [mc Username]:
-    entry $pca.euser -textvariable [namespace current]::tmpPrefs(httpproxyusername)
-    label $pca.lpass -text [mc Password]:
-    entry $pca.epass -textvariable [namespace current]::tmpPrefs(httpproxypassword)
-
-    grid $pca.msg   -          -sticky w
-    grid $pca.lserv $pca.eserv
-    grid $pca.lport $pca.eport
-    grid x          $pca.auth  -sticky w
-    grid $pca.luser $pca.euser
-    grid $pca.lpass $pca.epass
-    grid $pca.lserv $pca.lport $pca.luser $pca.lpass -sticky e
-    grid $pca.eserv $pca.eport $pca.euser $pca.epass -sticky ew
+    if {0} {
+	set pca $page.fr
+	labelframe $pca -text {Http Proxy} -padx 12 -pady 4
+	pack $pca -side top -anchor w -padx 8 -pady 4
+	
+	label $pca.msg -wraplength 300 -justify left \
+	  -text "Usage of the Http proxy is determined\
+	  by each profile settings. File transfers wont work if you use Http proxy!\
+	  NOT YET WORKING!"
+	
+	label $pca.lserv -text [mc {Proxy Server}]:
+	entry $pca.eserv -textvariable [namespace current]::tmpPrefs(httpproxyserver)
+	label $pca.lport -text [mc {Proxy Port}]:
+	entry $pca.eport -textvariable [namespace current]::tmpPrefs(httpproxyport)
+	checkbutton $pca.auth -text " [mc {Use proxy authorization}]" \
+	  -variable [namespace current]::tmpPrefs(httpproxyauth)
+	label $pca.luser -text [mc Username]:
+	entry $pca.euser -textvariable [namespace current]::tmpPrefs(httpproxyusername)
+	label $pca.lpass -text [mc Password]:
+	entry $pca.epass -textvariable [namespace current]::tmpPrefs(httpproxypassword)
+	
+	grid $pca.msg   -          -sticky w
+	grid $pca.lserv $pca.eserv
+	grid $pca.lport $pca.eport
+	grid x          $pca.auth  -sticky w
+	grid $pca.luser $pca.euser
+	grid $pca.lpass $pca.epass
+	grid $pca.lserv $pca.lport $pca.luser $pca.lpass -sticky e
+	grid $pca.eserv $pca.eport $pca.euser $pca.epass -sticky ew
+    }
 }
 
 proc ::Preferences::Proxies::SavePrefsHook { } {
@@ -753,19 +759,21 @@ proc ::Preferences::Proxies::UserDefaultsHook { } {
 #       its preference file.
 
 proc ::Preferences::SavePushBt { } {
-    global  prefs
+    global  prefs wDlgs
     
     variable wtoplevel
     variable finished
     variable tmpPrefs
     variable tmpJPrefs
+    variable needRestart
     
     set protocolSet 0
+    set needRestart 0
     
     # Was protocol changed?
     if {![string equal $prefs(protocol) $tmpPrefs(protocol)]} {
-	set ans [tk_messageBox -title Relaunch -icon info -type yesno \
-	  -message [FormatTextForMessageBox [mc messprotocolch]]]
+	set ans [::UI::MessageBox -title Relaunch -icon info -type yesno \
+	  -message [mc messprotocolch]]
 	if {$ans == "no"} {
 	    set finished 1
 	    return
@@ -780,13 +788,20 @@ proc ::Preferences::SavePushBt { } {
     
     # Let components store themselves.
     ::hooks::run prefsSaveHook
-	
+
+    if {$needRestart} {
+	set ans [::UI::MessageBox -title [mc Warning]  \
+	  -type ok -parent $wDlgs(prefs) -icon info \
+	  -message [mc messprefsrestart]]
+    }
+
     # Save the preference file.
     ::PreferencesUtils::SaveToFile
     ::Preferences::CleanUp
     
     set finished 1
     destroy $wtoplevel
+    ::hooks::run prefsDestroyHook
 }
 
 proc ::Preferences::CloseHook {wclose} {
@@ -849,9 +864,9 @@ proc ::Preferences::CancelPushBt { } {
     ::hooks::run prefsCancelHook
     
     if {$hasChanged} {
-	set ans [tk_messageBox -title [mc Warning]  \
+	set ans [::UI::MessageBox -title [mc Warning]  \
 	  -type yesno -default no -parent $wDlgs(prefs) -icon warning \
-	  -message [FormatTextForMessageBox [mc messprefschanged]]]
+	  -message [mc messprefschanged]]
 	if {$ans == "yes"} {
 	    set finished 2
 	}
@@ -859,8 +874,9 @@ proc ::Preferences::CancelPushBt { } {
 	set finished 2
     }
     if {$finished == 2} {
-	::Preferences::CleanUp
+	CleanUp
 	destroy $wtoplevel
+	::hooks::run prefsDestroyHook
     }
     return $ans
 }
@@ -890,7 +906,12 @@ proc ::Preferences::HasChanged { } {
     variable hasChanged
 
     set hasChanged 1
-    CallTrace 4
+}
+
+proc ::Preferences::NeedRestart { } {
+    variable needRestart
+
+    set needRestart 1
 }
 
 # Preferences::SelectCmd --
@@ -909,7 +930,13 @@ proc ::Preferences::SelectCmd {w v} {
     variable nbframe
 
     if {[llength $v] && ([$w itemconfigure $v -dir] == 0)} {
-	$nbframe displaypage [lindex $v end]
+	#$nbframe displaypage [lindex $v end]
+    }    
+    if {[llength $v]} {
+	set page [lindex $v end]
+	if {[$nbframe exists $page]} {
+	    $nbframe displaypage $page
+	}
     }    
 }
 
