@@ -7,7 +7,7 @@
 #  
 #  See the README file for license, bugs etc.
 #  
-# $Id: Utils.tcl,v 1.30 2004/08/28 07:00:08 matben Exp $
+# $Id: Utils.tcl,v 1.40 2005/01/09 14:33:52 matben Exp $
 
 namespace eval ::Utils:: {
 
@@ -66,6 +66,33 @@ proc lrevert {args} {
     }
     return $tmp
 }
+
+# listintersect --
+# 
+#       Intersections of two lists.
+
+proc listintersect {alist blist} {
+    set tmp {}
+    foreach a $alist {
+	if {[lsearch $blist $a] >= 0} {
+	    lappend tmp $a
+	}
+    }
+    return $tmp
+}
+
+# listintersectnonempty --
+# 
+#       Is intersection of two lists non empty.
+
+proc listintersectnonempty {alist blist} {
+    foreach a $alist {
+	if {[lsearch $blist $a] >= 0} {
+	    return 1
+	}
+    }
+    return 0
+}
     
 # getdirname ---
 #
@@ -86,37 +113,6 @@ proc getdirname {filePath} {
     }
 }
 
-# FormatTextForMessageBox --
-#
-#       The tk_messageBox needs explicit newlines to format the message text.
-
-proc FormatTextForMessageBox {txt {width ""}} {
-    global  prefs
-
-    if {[string equal $::tcl_platform(platform) "windows"]} {
-
-	# Insert newlines to force line breaks.
-	if {[string length $width] == 0} {
-	    set width $prefs(msgWrapLength)
-	}
-	set len [string length $txt]
-	set start $width
-	set first 0
-	set newtxt {}
-	while {([set ind [tcl_wordBreakBefore $txt $start]] > 0) &&  \
-	  ($start < $len)} {	    
-	    append newtxt [string trim [string range $txt $first [expr $ind-1]]]
-	    append newtxt "\n"
-	    set start [expr $ind + $width]
-	    set first $ind
-	}
-	append newtxt [string trim [string range $txt $first end]]
-	return $newtxt
-    } else {
-	return $txt
-    }
-}
-
 # SecondCoccinella --
 # 
 #       This gets called if a second instance of this app is launched,
@@ -239,7 +235,8 @@ proc ::Utils::GetDomainNameFromUrl {url} {
 #       Pretty formatted time & date.
 #
 # Arguments:
-#       secs        number of seconds since system defined time.
+#       secs        Number of seconds since system defined time.
+#                   This must be local time.
 #       
 # Results:
 #       nice time string that still can be used by 'clock scan'
@@ -248,11 +245,13 @@ proc ::Utils::SmartClockFormat {secs args} {
     
     array set opts {
 	-weekdays 0
+	-detail   0
     }
     array set opts $args
     
     # 'days': 0=today, -1=yesterday etc.
-    set days [expr ($secs - [clock scan "today 00:00"])/(60*60*24)]
+    set secs00 [clock scan "today 00:00"]
+    set days [expr ($secs - $secs00)/(60*60*24)]
     
     switch -regexp -- $days {
 	^1$ {
@@ -266,9 +265,8 @@ proc ::Utils::SmartClockFormat {secs args} {
 	}
 	^-[2-5]$ {
 	    if {$opts(-weekdays)} {
-		# clock scan doesn't work on these
 		set date [string tolower  \
-		  [clock format [clock scan "today $days days"] -format "%A"]]
+		  [clock format [clock scan "$days days ago"] -format "%A"]]
 	    } else {
 		set date [clock format $secs -format "%y-%m-%d"]
 	    }
@@ -277,55 +275,99 @@ proc ::Utils::SmartClockFormat {secs args} {
 	    set date [clock format $secs -format "%y-%m-%d"]
 	}
     }
-    
-    set time [clock format $secs -format "%H:%M:%S"]
+    if {$opts(-detail) && ($days == 0)} {
+	set now [clock seconds]
+	set minutes [expr ($now - $secs)/60]
+	if {$minutes == 0} {
+	    set time [clock format $secs -format "%H:%M:%S"]
+	} elseif {$minutes == 1} {
+	    set time "one minute ago"
+	} elseif {$minutes == 2} {
+	    set time "two minutes ago"
+	} elseif {$minutes == 3} {
+	    set time "three minutes ago"
+	} elseif {$minutes < 60} {
+	    set time "$minutes minutes ago"
+	} else {
+	    set time [clock format $secs -format "%H:%M:%S"]
+	}
+    } else {	
+	set time [clock format $secs -format "%H:%M:%S"]
+    }
     return "$date $time"
 }
 
+proc ::Utils::IsToday {secs} {
+    return [expr ($secs - [clock scan "today 00:00"])/(60*60*24) >= 0 ? 1 : 0]
+}
+
 proc ::Utils::UnixGetWebBrowser { } {
-    global  this prefs
+    global  this prefs env
     
     set browser ""
     if {$this(platform) == "unix"} {
-
-	# Try in order.
-	set found 0
-	set browsers [list $prefs(webBrowser) netscape mozilla konqueror]
-	foreach app $browsers {
-	    if {![catch {exec which $app}]} {
-		set prefs(webBrowser) $app
-		set browser $app
-		break
+	if {[info exists env(BROWSER)]} {
+	    if {[llength [auto_execok $env(BROWSER)]] > 0} {
+		set browser $env(BROWSER)
 	    }
 	}
+	set cmd [auto_execok $prefs(webBrowser)]
+	if {$cmd == {}} {
+	    foreach name {firefox galeon konqueror mozilla-firefox \
+	      mozilla-firebird mozilla netscape iexplorer opera} {
+		if {[llength [set e [auto_execok $name]]] > 0} {
+		    set browser [lindex $e 0]
+		    break
+		}
+	    }
+	}
+	set prefs(webBrowser) $browser
     }
     return $browser
 }
 
+proc ::Utils::UnixOpenUrl {url} {
+    global  prefs
+
+    if {$prefs(webBrowser) == ""} {
+	set browser [UnixGetWebBrowser]
+    } else {
+	set browser $prefs(webBrowser)
+    }
+    if {$browser != ""} {
+	if {[catch {eval exec $browser -remote \"openURL($url, new-tab)\"}]} {
+	    if {[catch {exec $browser -remote $url}]} {
+		if {[catch {exec $browser $url &}]} {
+		    set browser ""
+		}
+	    }
+	}
+    } 
+    if {$browser == ""} {
+	::UI::MessageBox -icon error -type ok -message \
+	  "Couldn't localize a web browser on this system.\
+	  Define a shell variable env(BROWSER) to point to a web browser."
+    }
+}
+
 proc ::Utils::OpenURLInBrowser {url} {
-    global  this prefs
+    global  tcl_platform prefs
     
     ::Debug 2 "::Utils::OpenURLInBrowser url=$url"
     
-    switch $this(platform) {
-	unix {
-	    set browser [::Utils::UnixGetWebBrowser]
-	    if {$browser == ""} {
-		tk_messageBox -icon error -type ok -message \
-		  "Couldn't localize a web browser on this system"
-	    } else {
-		exec $browser $url &
-	    }
+    switch -glob -- $tcl_platform(platform),$tcl_platform(os) {
+	unix,Darwin {
+	    exec open $url
 	}
-	windows {	    
+	unix,* {
+	    UnixOpenUrl $url
+	}
+	windows,* {	    
 	    ::Windows::OpenUrl $url
 	}
-	macintosh {    
+	macintosh,* {    
 	    ::Mac::OpenUrl $url
 	}
-	macosx {
-	    exec open $url
-	}
     }
 }
 
@@ -343,6 +385,8 @@ proc ::Utils::ValidMinutes {str} {
 #
 #       Makes a unique hex string stamped by time.
 #       Can generate max 'maxuidpersec' (uniques) uid's per second.
+#       
+#       See also ::tfileutils::newuid !!!
 
 proc ::Utils::GenerateHexUID { } {
     variable uid
@@ -393,71 +437,12 @@ proc ::Utils::GetFontListFromName {fontSpec} {
 proc ::Utils::GetHttpFromFile {filePath} {
     global  prefs this
     
-    set relPath [filerelative $this(httpdRootPath) $filePath]
+    set relPath [::tfileutils::relative $this(httpdRootPath) $filePath]
     set relPath [uriencode::quotepath $relPath]
     set ip [::Network::GetThisPublicIPAddress]
     return "http://${ip}:$prefs(httpdPort)/$relPath"
 }
 
-# Utils::ProgressWindow, ProgressFree --
-# 
-#       Useful when using progress window. It combines the ProgressWindow
-#       with Timing.
-
-namespace eval ::Utils:: {
-    
-    variable puid 0
-}
-
-proc ::Utils::ProgressWindow {token total current args} {
-    global  prefs wDlgs tcl_platform
-    variable progress
-    variable puid
-   
-    # Cache timing info.
-    ::Timing::Set $token $current
-
-    # Update only when minimum time has passed, and only at certain interval.
-    set ms [clock clicks -milliseconds]
-    set needupdate 0
-
-    # Create progress dialog if not exists.
-    if {![info exists progress($token,w)]} {
-	set progress($token,token) $token
-	set w $wDlgs(prog)2[incr puid]
-	set progress($token,w) $w
-	eval {::ProgressWindow::ProgressWindow $w} $args
-	set progress($token,startmillis) $ms
-	set progress($token,lastmillis)  $ms
-	set needupdate 1
-    } elseif {[expr $ms - $progress($token,lastmillis)] > $prefs(progUpdateMillis)} {
-
-	# Update the progress window.
-	append msg3 "[mc Rate]: [::Timing::FormMessage $token $total]"	
-	set percent [expr 100.0 * $current/($total + 0.001)]
-	$progress($token,w) configure -percent $percent -text3 $msg3
-	set progress($token,lastmillis) $ms
-	set needupdate 1
-    }
-
-    # Be silent... except for a necessary update command to not block.
-    if {$needupdate} {
-	if {[string equal $tcl_platform(platform) "windows"]} {
-	    update
-	} else {
-	    update idletasks
-	}
-    }
-}
-
-proc ::Utils::ProgressFree {token} {
-    variable progress
-    
-    catch {destroy $progress($token,w)}
-    ::Timing::Reset $token
-    array unset progress $token,*
-}
-
 #--- Animation -----------------------------------------------------------------
 
 namespace eval ::Utils:: {
@@ -506,269 +491,112 @@ proc ::Utils::AnimateStop {uid} {
     array unset anim $uid,*
 }
 
-#--- Timing --------------------------------------------------------------------
-
-namespace eval ::Timing:: {
-    variable timing
-}
-
-# Timing::Set, Reset, GetRate, FormMessage --
-# 
-#       A number of utils that handle timing objects. Mainly to get bytes
-#       per second during file transfer.
-#       
-# Arguments:
-#       key         a unique key to identify a particular timing object,
-#                   typically use the socket token or a running namespaced 
-#                   number.
-#       bytes       number of bytes transported so far
-#       totalbytes  total file size in bytes
-
-proc ::Timing::Set {key bytes} {
-    variable timing
-    
-    lappend timing($key) \
-      [list [expr double([clock clicks -milliseconds])] $bytes]
-    return ""
-}
-
-proc ::Timing::Reset {key} {
-    variable timing
-    
-    unset -nocomplain timing($key)
-}
-
-proc ::Timing::GetRate {key} {
-    variable timing
-	
-    set len [llength $timing($key)]
-    if {$len <= 1} {
-	return 0.0
-    }
-    set nAve 12
-    set istart [expr $len - $nAve]
-    if {$istart < 0} {
-	set istart 0
-    }
-    
-    # Keep only the part we are interested in.
-    set timing($key) [lrange $timing($key) $istart end]
-    set timeList $timing($key)
-    set sumMillis [expr [lindex $timeList end 0] - [lindex $timeList 0 0]]
-    set sumBytes [expr [lindex $timeList end 1] - [lindex $timeList 0 1]]
-    
-    # Treat the case with wrap around. (Guess)
-    if {$sumMillis <= 0} {
-	set sumMillis 1000000
-    }
-    
-    # Returns average bytes per second.
-    return [expr 1000.0 * $sumBytes / ($sumMillis + 1.0)]
-}
-
-proc ::Timing::GetRateLinearInterp {key} {
-    variable timing
-    
-    set len [llength $timing($key)]
-    if {$len <= 1} {
-	return 0.0
-    }
-    set n 12
-    set istart [expr $len - $n]
-    if {$n > $len} {
-	set n $len
-	set istart 0
-    }
-    
-    # Keep only the part we are interested in.
-    set timing($key) [lrange $timing($key) $istart end]
-    set sumx  0.0
-    set sumy  0.0
-    set sumxy 0.0
-    set sumx2 0.0
-    
-    # Need to move origin to get numerical stability!
-    set x0 [lindex $timing($key) 0 0]
-    set y0 [lindex $timing($key) 0 1]
-    foreach co $timing($key) {
-	set x [expr [lindex $co 0] - $x0]
-	set y [expr [lindex $co 1] - $y0]
-	set sumx  [expr $sumx + $x]
-	set sumy  [expr $sumy + $y]
-	set sumxy [expr $sumxy + $x * $y]
-	set sumx2 [expr $sumx2 + $x * $x]
-    }
-    
-    # This is bytes per millisecond.
-    set k [expr ($n * $sumxy - $sumx * $sumy) /  \
-      ($n * $sumx2 - $sumx * $sumx)]
-    return [expr 1000.0 * $k]
-}
-
-proc ::Timing::GetPercent {key totalbytes} {
-    variable timing
-
-    if {[llength $timing($key)] > 1} {
-	set bytes [lindex $timing($key) end 1]
-    } else {
-	set bytes 0
-    }
-    set percent [format "%3.0f" [expr 100.0 * $bytes/($totalbytes + 1.0)]]
-    set percent [expr $percent < 0 ? 0 : $percent]
-    set percent [expr $percent > 100 ? 100 : $percent]
-    return $percent
-}
-
-proc ::Timing::FormMessage {key totalbytes} {
-    variable timing
-    
-    #set bytesPerSec [::Timing::GetRateLinearInterp $key]
-    set bytesPerSec [::Timing::GetRate $key]
-
-    # Find format: bytes or k.
-    if {$bytesPerSec < 1000} {
-	set txtRate "[expr int($bytesPerSec)] bytes/sec"
-    } elseif {$bytesPerSec < 1000000} {
-	set txtRate [list [format "%.1f" [expr $bytesPerSec/1000.0] ]Kb/sec]
-    } else {
-	set txtRate [list [format "%.1f" [expr $bytesPerSec/1000000.0] ]Mb/sec]
-    }
-
-    # Remaining time.
-    if {[llength $timing($key)] > 1} {
-	set bytes [lindex $timing($key) end 1]
-    } else {
-	set bytes 0
-    }
-    set percent [format "%3.0f" [expr 100.0 * $bytes/($totalbytes + 1.0)]]
-    set secsLeft  \
-      [expr int(ceil(($totalbytes - $bytes)/($bytesPerSec + 1.0)))]
-    if {$secsLeft < 60} {
-	set txtTimeLeft ", $secsLeft secs remaining"
-    } elseif {$secsLeft < 120} {
-	set txtTimeLeft ", one minute and [expr $secsLeft - 60] secs remaining"
-    } else {
-	set txtTimeLeft ", [expr $secsLeft/60] minutes remaining"
-    }
-    return "${txtRate}${txtTimeLeft}"
-}
-
 #--- Utilities for the Text widget ---------------------------------------------
 
 namespace eval ::Text:: {
 
-    # Unique counter to produce http link tags.
-    variable numLink 1000
-    
     # Unique counter to produce specified link tags.
     variable idurl 1000
-    
-    # Storage for mapping idurl -> url
-    variable idToUrlArr
+        
+    variable urlRegexp {^(http://|https://|www\.|ftp://|ftp\.)([^ \t\r\n]+)}
+    variable urlColor
+    array set urlColor {fg blue activefg red}
 }
 
-# Text::URLLabel --
-#
-#       An url widget.
-
-proc ::Text::URLLabel {w url args} {
-    
-    array set opts [list -height 1 -width [string length $url] -bd 0   \
-      -wrap word -highlightthickness 0]
-    array set opts $args
-    eval {text $w} [array get opts]
-    $w tag configure normal -foreground blue -underline 1
-    $w tag configure active -foreground red -underline 1   
-    $w tag bind normal <Enter> [list ::Text::EnterLink $w %x %y normal active]
-    $w tag bind active <ButtonPress>  \
-      [list ::Text::ButtonPressOnLink $w %x %y active]
-    $w insert end $url normal
-    return $w
-}
-
-# Text::ParseHttpLinks --
+# Text::Parse, ... --
 # 
-#       Parses text translating elements that may be interpreted as an url
-#       to a clickable thing.
-#
-# Arguments:
-#       str         the text string, tcl special chars already protected
-#       tag         the normal text tag
-#       linktag     the tag for links
-#       
-# Results:
-#       A list {textcmd textcmd ...} where textcmd is typically:
-#       "insert end {Some text} $tag"
+#       It takes a text widget, the text, and a default tag, and parses
+#       smileys and urls.
 
-proc ::Text::ParseHttpLinks {str tag linktag} {
+proc ::Text::Parse {w str tag} {
     
-    # Protect all  *regexp*  special characters.
-    #regsub -all {\\|&} $str {\\\0} str
-
-    # regexp hell, welcome!
-    set wsp_ "\[ \t\r\n]"
-    set path_ "\[^ \t\r\n,]"
-    set epath_ "\[^ \t\r\n,\.]"
-    set start_ "(^|\"|$wsp_)"
-    set end_ "(\$|\"|$wsp_)"
-
-    # This is extremely tricky business!
-    # Character data must not be embraced, but must be qoted in order for
-    # the protected tcl special chars to be deprotected.
-    set re "${start_}((http://|www\\.)${path_}+${epath_})"
-    set sub "\\1\" $tag\} \{insert end \{\\2\} \{$tag $linktag\}\} \{insert end \""
-    regsub -all -nocase -- $re $str $sub txtlist
-    return "\{insert end \"$txtlist\" $tag\}"
-}
-
-proc ::Text::ConfigureLinkTagForTextWidget {w tag linkactive} {
-    
-    $w tag configure $tag -foreground blue -underline 1
-    $w tag configure $linkactive -foreground red -underline 1
-    
-    $w tag bind $tag <Enter> [list ::Text::EnterLink $w %x %y $tag $linkactive]
-    $w tag bind $linkactive <ButtonPress>  \
-      [list ::Text::ButtonPressOnLink $w %x %y $linkactive]
-}
-
-proc ::Text::EnterLink {w x y linktag linkactive} {
-    
-    set range [$w tag prevrange $linktag "@$x,$y +1 char"]
-    if {[llength $range]} {
-	eval {$w tag add $linkactive} $range
-	eval {$w tag remove $linktag} $range
-	$w configure -cursor hand2
-	$w tag bind $linkactive <Leave>  \
-	  [list ::Text::LeaveLink $w $linktag $linkactive]
+    # Split string into words and whitespaces.
+    set wsp {[ \t\r\n]+}
+    set len [string length $str]
+    if {$len == 0} {
+	return
     }
-}
-
-proc ::Text::LeaveLink {w linktag linkactive} {
-    
-    set range [$w tag ranges $linkactive]
-    if {[llength $range]} {
-	eval {$w tag add $linktag} $range
-	eval {$w tag remove $linkactive} $range
-    }
-    $w configure -cursor arrow
-}
-
-proc ::Text::ButtonPressOnLink {w x y linkactive} {
-    
-    ::Debug 2 "::Text::ButtonPressOnLink"
-
-    set range [$w tag prevrange $linkactive "@$x,$y"]
-    if {[llength $range]} {
-	set url [string trim [eval $w get $range]]
+    set start 0
+    while {[regexp -start $start -indices -- $wsp $str match]} {
+	foreach {matchStart matchEnd} $match break
 	
-	# Add "http://" if not there.
-	if {![regexp {^http://.+} $url]} {
-	    set url "http://$url"
-	}
-	if {[::Utils::IsWellformedUrl $url]} {
-	    ::Utils::OpenURLInBrowser $url
-	}
+	# The "space" part.
+	set space [string range $str $matchStart $matchEnd]
+	incr matchStart -1
+	incr matchEnd
+	
+	# The word preceeding the space.
+	set word [string range $str $start $matchStart]
+	set start $matchEnd
+	
+	# Process the actual word.
+	ParseWord $w $word $tag
+	
+	# Insert the whitespace after word.
+	$w insert end $space $tag
     }
+    
+    # And the final word.
+    set word [string range $str $start end]
+    ParseWord $w $word $tag
+}
+
+proc ::Text::ParseWord {w word tag} {
+    
+    if {[::Emoticons::Exists $word]} {
+	::Emoticons::Make $w $word
+    } elseif {![ParseUrl $w $word]} {
+	$w insert end $word $tag
+    }
+}
+
+proc ::Text::ParseUrl {w word} {
+    variable urlRegexp
+    variable idurl
+    variable urlColor
+    
+    if {[regexp $urlRegexp $word]} {
+	set urltag url${idurl}
+	set urlfg       [option get $w urlForeground       Text]
+	set urlactivefg [option get $w urlActiveForeground Text]
+	if {$urlfg == ""} {
+	    set urlfg $urlColor(fg)
+	}
+	if {$urlactivefg == ""} {
+	    set activefg $urlColor(activefg)
+	}
+	$w tag configure $urltag -foreground $urlfg -underline 1
+	$w tag bind $urltag <Button-1>  \
+	  [list ::Text::UrlButton [string map {% %%} $word]]
+	$w tag bind $urltag <Any-Enter>  \
+	  [list ::Text::UrlEnter $w $urltag $activefg]
+	$w tag bind $urltag <Any-Leave>  \
+	  [list ::Text::UrlLeave $w $urltag $urlfg]
+	$w insert end $word $urltag
+	incr idurl
+	return 1
+    } else {
+	return 0
+    }
+}
+
+proc ::Text::UrlButton {url} {
+    if {![regexp {^http://.+} $url]} {
+	set url "http://$url"
+    }
+    if {[::Utils::IsWellformedUrl $url]} {
+	::Utils::OpenURLInBrowser $url
+    }
+}
+
+proc ::Text::UrlEnter {w tag fgcol} {
+    $w configure -cursor hand2
+    $w tag configure $tag -foreground $fgcol -underline 1
+}
+
+proc ::Text::UrlLeave {w tag fgcol} {
+    $w configure -cursor arrow
+    $w tag configure $tag -foreground $fgcol -underline 1
 }
 
 # Text::InsertURL --
@@ -777,39 +605,28 @@ proc ::Text::ButtonPressOnLink {w x y linkactive} {
 
 proc ::Text::InsertURL {w str url tag} {    
     variable idurl
-    variable idToUrlArr
-
-    set idToUrlArr($idurl) $url
+    variable urlColor
+    
     set urltag url${idurl}
-    set linkactive ${urltag}active
+    set urlfg       [option get $w urlForeground       Text]
+    set urlactivefg [option get $w urlActiveForeground Text]
+    if {$urlfg == ""} {
+	set urlfg $urlColor(fg)
+    }
+    if {$urlactivefg == ""} {
+	set activefg $urlColor(activefg)
+    }
+    $w tag configure $urltag -foreground $urlfg -underline 1
+    $w tag bind $urltag <Button-1>  \
+      [list ::Text::UrlButton [string map {% %%} $url]]
+    $w tag bind $urltag <Any-Enter>  \
+      [list ::Text::UrlEnter $w $urltag $activefg]
+    $w tag bind $urltag <Any-Leave>  \
+      [list ::Text::UrlLeave $w $urltag $urlfg]
     $w insert end $str [list $tag $urltag]
-
-    $w tag configure $urltag -foreground blue -underline 1
-    $w tag configure $linkactive -foreground red -underline 1
-    
-    $w tag bind $urltag <Enter>  \
-      [list ::Text::EnterLink $w %x %y $urltag $linkactive]
-    $w tag bind $linkactive <ButtonPress>  \
-      [list ::Text::ButtonPressOnURL $w $idurl]
-
     incr idurl
 }
 
-proc ::Text::ButtonPressOnURL {w idurl} {    
-    variable idToUrlArr
-
-    set url $idToUrlArr($idurl)
-    ::Debug 2 "::Text::ButtonPressOnURL url=$url"
-    
-    # Add "http://" if not there.
-    if {![regexp {^http://.+} $url]} {
-	set url "http://$url"
-    }
-    if {[::Utils::IsWellformedUrl $url]} {
-	::Utils::OpenURLInBrowser $url
-    }
-}
-
 proc ::Text::TransformToPureText {w args} {    
     variable puretext
     
