@@ -12,7 +12,7 @@
 #  
 #  See the README file for license, bugs etc.
 #  
-# $Id: Plugins.tcl,v 1.12 2004/09/28 13:50:21 matben Exp $
+# $Id: Plugins.tcl,v 1.15 2004/12/08 08:21:20 matben Exp $
 #
 # We need to be very systematic here to handle all possible MIME types
 # and extensions supported by each package or helper application.
@@ -86,6 +86,7 @@ namespace eval ::Plugins:: {
     ::hooks::register prefsUserDefaultsHook  ::Plugins::UserDefaultsHook
     ::hooks::register prefsSaveHook          ::Plugins::SaveHook
     ::hooks::register prefsCancelHook        ::Plugins::CancelHook
+    ::hooks::register prefsDestroyHook       ::Plugins::DestroyPrefsHook
     ::hooks::register initHook               ::Plugins::InitHook
 
     variable inited 0
@@ -406,7 +407,8 @@ proc ::Plugins::CompileAndLoadPackages { } {
     # And all helper applications... only apps on Unix/Linux.
     foreach helperApp $plugin(allApps) {
 	if {[string equal $this(platform) "unix"]} {
-	    if {![catch {exec which $helperApp} apath]} {
+	    set apath [lindex [auto_execok $helperApp] 0]
+	    if {[llength $apath]} {
 		set plugin($helperApp,loaded) 1
 	    } else  {
 		set plugin($helperApp,loaded) 0
@@ -1284,6 +1286,9 @@ proc ::Plugins::InitPrefsHook { } {
 
 proc ::Plugins::BuildPrefsHook {wtree nbframe} {
     
+    if {![$wtree isitem Whiteboard]} {
+	$wtree newitem {Whiteboard} -text [mc Whiteboard]
+    }
     $wtree newitem {Whiteboard Plugins2} -text [mc Plugins]
 
     set wpage [$nbframe page Plugins2]
@@ -1306,7 +1311,7 @@ proc ::Plugins::BuildPrefsPage {page} {
     label ${pbl}.lhead -wraplength 300 -anchor w -justify left \
       -text [mc prefplugctrl]
     pack ${pbl}.lhead -padx 0 -pady 2 -side top -anchor w
-    
+        
     set pfr [frame ${pbl}.frpl]
     pack $pfr -side top -anchor w
     set i 0
@@ -1323,21 +1328,28 @@ proc ::Plugins::BuildPrefsPage {page} {
 	grid ${pfr}.c${i} -row $i -column 1 -padx 2 -sticky ew
 	incr i
     }
+    foreach plug $prefs(pluginBanList) {
+	set tmpPrefPlugins($plug) 0
+	set prefplugins($plug) $tmpPrefPlugins($plug)
+    }
 }
 
 proc ::Plugins::SaveHook { } {
     global prefs
+    variable prefplugins
     variable tmpPrefPlugins
-
+    
     # To be correct we should also have loaded the pack here. TODO.
     set banList {}
     foreach name [array names tmpPrefPlugins] {
 	if {$tmpPrefPlugins($name) == 0} {
 	    lappend banList $name
 	}
+	if {$tmpPrefPlugins($name) != $prefplugins($name)} {
+	    ::Preferences::NeedRestart
+	}
     }
     set prefs(pluginBanList) [lsort -unique $banList]
-    unset -nocomplain tmpPrefPlugins
 }
 
 proc ::Plugins::CancelHook { } {
@@ -1363,4 +1375,11 @@ proc ::Plugins::UserDefaultsHook { } {
     }    
 }
 
+proc ::Plugins::DestroyPrefsHook { } {
+    variable prefplugins
+    variable tmpPrefPlugins
+    
+    unset -nocomplain tmpPrefPlugins
+}
+
 #-------------------------------------------------------------------------------
